#include "sound_effects.hpp"
#include "settings.hpp"
#include <cstdlib>
#include <stdint.h>

namespace SFX {
    const std::array<SeqType, SFX_COUNT> seqTypesSFX = {
        /* NA_SE_PL_WALK_GROUND */              SEQ_WALK,
        /* NA_SE_PL_WALK_SAND */                SEQ_WALK,
        /* NA_SE_PL_WALK_CONCRETE */            SEQ_WALK,
        /* NA_SE_PL_WALK_DIRT */                SEQ_WALK,
        /* NA_SE_PL_WALK_WATER0 */              SEQ_WALK,
        /* NA_SE_PL_WALK_WATER1 */              SEQ_WALK,
        /* NA_SE_PL_WALK_WATER2 */              SEQ_WALK, // Tall grass?
        /* NA_SE_PL_WALK_MAGMA */               SEQ_WALK,
        /* NA_SE_PL_WALK_GRASS */               SEQ_WALK,
        /* NA_SE_PL_WALK_IRON */                SEQ_WALK,
        /* NA_SE_PL_WALK_LADDER */              SEQ_WALK,
        /* NA_SE_PL_WALK_GLASS */               SEQ_WALK, // Planks?
        /* NA_SE_PL_WALK_WALL */                SEQ_WALK,
        /* NA_SE_PL_WALK_HEAVYBOOTS */          SEQ_WALK_LOUD,
        /* NA_SE_PL_DUMMY_14 */                 SEQ_WALK, // Snow?
        /* NA_SE_PL_WALK_ICE */                 SEQ_WALK,
        /* NA_SE_PL_JUMP */                     SEQ_JUMP,
        /* NA_SE_PL_JUMP_SAND */                SEQ_JUMP,
        /* NA_SE_PL_JUMP_CONCRETE */            SEQ_JUMP,
        /* NA_SE_PL_JUMP_DIRT */                SEQ_JUMP,
        /* NA_SE_PL_JUMP_WATER0 */              SEQ_JUMP,
        /* NA_SE_PL_JUMP_WATER1 */              SEQ_JUMP,
        /* NA_SE_PL_JUMP_WATER2 */              SEQ_JUMP,
        /* NA_SE_PL_JUMP_MAGMA */               SEQ_JUMP,
        /* NA_SE_PL_JUMP_GRASS */               SEQ_JUMP,
        /* NA_SE_PL_JUMP_IRON */                SEQ_JUMP,
        /* NA_SE_PL_JUMP_LADDER */              SEQ_JUMP,
        /* NA_SE_PL_JUMP_GLASS */               SEQ_JUMP,
        /* NA_SE_PL_DUMMY28 */                  SEQ_JUMP,
        /* NA_SE_PL_JUMP_HEAVYBOOTS */          SEQ_NOCAT,
        /* NA_SE_PL_DUMMY30 */                  SEQ_JUMP,
        /* NA_SE_PL_JUMP_ICE */                 SEQ_JUMP,
        /* NA_SE_PL_LAND */                     SEQ_LAND,
        /* NA_SE_PL_LAND_SAND */                SEQ_LAND,
        /* NA_SE_PL_LAND_CONCRETE */            SEQ_LAND,
        /* NA_SE_PL_LAND_DIRT */                SEQ_LAND,
        /* NA_SE_PL_LAND_WATER0 */              SEQ_LAND,
        /* NA_SE_PL_LAND_WATER1 */              SEQ_LAND,
        /* NA_SE_PL_LAND_WATER2 */              SEQ_LAND,
        /* NA_SE_PL_LAND_MAGMA */               SEQ_LAND,
        /* NA_SE_PL_LAND_GRASS */               SEQ_LAND,
        /* NA_SE_PL_LAND_IRON */                SEQ_LAND,
        /* NA_SE_PL_LAND_LADDER */              SEQ_LAND,
        /* NA_SE_PL_LAND_GLASS */               SEQ_LAND,
        /* NA_SE_PL_DUMMY_44 */                 SEQ_LAND,
        /* NA_SE_PL_LAND_HEAVYBOOTS */          SEQ_NOCAT,
        /* NA_SE_PL_DUMMY_46 */                 SEQ_LAND,
        /* NA_SE_PL_LAND_ICE */                 SEQ_LAND,
        /* NA_SE_PL_SLIPDOWN */                 SEQ_NOCAT,
        /* NA_SE_PL_CLIMB_CLIFF */              SEQ_NOCAT,
        /* NA_SE_PL_SIT_ON_HORSE */             SEQ_NOCAT,
        /* NA_SE_PL_GET_OFF_HORSE */            SEQ_NOCAT,
        /* NA_SE_PL_TAKE_OUT_SHIELD */          SEQ_NOCAT,
        /* NA_SE_PL_CHANGE_ARMS */              SEQ_NOCAT,
        /* NA_SE_PL_CATCH_BOOMERANG */          SEQ_NOCAT,
        /* NA_SE_PL_DUMMY_55 */                 SEQ_NOSHUFFLE,
        /* NA_SE_PL_DUMMY_56 */                 SEQ_NOSHUFFLE,
        /* NA_SE_PL_SWIM */                     SEQ_NOCAT,
        /* NA_SE_PL_THROW */                    SEQ_NOCAT,
        /* NA_SE_PL_BODY_BOUND */               SEQ_NOCAT,
        /* NA_SE_PL_ROLL */                     SEQ_NOCAT,
        /* NA_SE_PL_SKIP */                     SEQ_NOCAT,
        /* NA_SE_PL_BODY_HIT */                 SEQ_NOCAT,
        /* NA_SE_PL_DAMAGE */                   SEQ_NOCAT,
        /* NA_SE_PL_SLIP */                     SEQ_SLIP,
        /* NA_SE_PL_SLIP_SAND */                SEQ_SLIP,
        /* NA_SE_PL_SLIP_CONCRETE */            SEQ_SLIP,
        /* NA_SE_PL_SLIP_DIRT */                SEQ_SLIP,
        /* NA_SE_PL_SLIP_WATER0 */              SEQ_SLIP,
        /* NA_SE_PL_SLIP_WATER1 */              SEQ_SLIP,
        /* NA_SE_PL_SLIP_WATER2 */              SEQ_SLIP,
        /* NA_SE_PL_SLIP_MAGMA */               SEQ_SLIP,
        /* NA_SE_PL_SLIP_GRASS */               SEQ_SLIP,
        /* NA_SE_PL_SLIP_IRON */                SEQ_SLIP,
        /* NA_SE_PL_SLIP_LADDER */              SEQ_SLIP,
        /* NA_SE_PL_SLIP_GLASS */               SEQ_SLIP,
        /* NA_SE_PL_DUMMY76 */                  SEQ_NOSHUFFLE,
        /* NA_SE_PL_SLIP_HEAVYBOOTS */          SEQ_SLIP,
        /* NA_SE_PL_DUMMY78 */                  SEQ_SLIP,
        /* NA_SE_PL_SLIP_ICE */                 SEQ_SLIP,
        /* NA_SE_PL_BOUND */                    SEQ_BOUND,
        /* NA_SE_PL_BOUND_SAND */               SEQ_BOUND,
        /* NA_SE_PL_BOUND_CONCRETE */           SEQ_BOUND,
        /* NA_SE_PL_BOUND_DIRT */               SEQ_BOUND,
        /* NA_SE_PL_BOUND_WATER0 */             SEQ_BOUND,
        /* NA_SE_PL_BOUND_WATER1 */             SEQ_BOUND,
        /* NA_SE_PL_BOUND_WATER2 */             SEQ_BOUND,
        /* NA_SE_PL_BOUND_MAGMA */              SEQ_BOUND,
        /* NA_SE_PL_BOUND_GRASS */              SEQ_BOUND,
        /* NA_SE_PL_BOUND_IRON */               SEQ_BOUND,
        /* NA_SE_PL_BOUND_LADDER */             SEQ_BOUND,
        /* NA_SE_PL_BOUND_WOOD */               SEQ_BOUND,
        /* NA_SE_PL_DUMMY_92 */                 SEQ_NOSHUFFLE,
        /* NA_SE_PL_BOUND_HEAVYBOOTS */         SEQ_BOUND,
        /* NA_SE_PL_DUMMY_94 */                 SEQ_BOUND,
        /* NA_SE_PL_BOUND_ICE */                SEQ_BOUND,
        /* NA_SE_PL_DUMMY_96 */                 SEQ_NOSHUFFLE,
        /* NA_SE_PL_DUMMY_97 */                 SEQ_NOSHUFFLE,
        /* NA_SE_PL_DUMMY_98 */                 SEQ_NOSHUFFLE,
        /* NA_SE_PL_FACE_UP */                  SEQ_NOCAT,
        /* NA_SE_PL_DIVE_BUBBLE */              SEQ_NOCAT,
        /* NA_SE_PL_MOVE_BUBBLE */              SEQ_NOCAT,
        /* NA_SE_PL_METALEFFECT_KID */          SEQ_NOCAT,
        /* NA_SE_PL_METALEFFECT_ADULT */        SEQ_NOCAT,
        /* NA_SE_PL_SPARK */                    SEQ_NOCAT,
        /* NA_SE_PL_PULL_UP_PLANT */            SEQ_NOCAT,
        /* NA_SE_PL_PULL_UP_ROCK */             SEQ_NOCAT,
        /* NA_SE_PL_IN_BUBBLE */                SEQ_NOCAT,
        /* NA_SE_PL_PULL_UP_BIGROCK */          SEQ_NOCAT,
        /* NA_SE_PL_SWORD_CHARGE */             SEQ_NOCAT,
        /* NA_SE_PL_FREEZE */                   SEQ_NOCAT,
        /* NA_SE_PL_PULL_UP_POT */              SEQ_NOCAT,
        /* NA_SE_PL_KNOCK */                    SEQ_NOCAT,
        /* NA_SE_PL_CALM_HIT */                 SEQ_NOCAT,
        /* NA_SE_PL_CALM_PAT */                 SEQ_NOCAT,
        /* NA_SE_PL_SUBMERGE */                 SEQ_NOCAT,
        /* NA_SE_PL_FREEZE_S */                 SEQ_NOCAT,
        /* NA_SE_PL_ICE_BROKEN */               SEQ_NOCAT,
        /* NA_SE_PL_SLIP_ICE_LELEL */           SEQ_SLIP_LOOP,
        /* NA_SE_PL_PUT_OUT_ITEM */             SEQ_NOCAT,
        /* NA_SE_PL_PULL_UP_WOODBOX */          SEQ_NOCAT,
        /* NA_SE_PL_MAGIC_FIRE */               SEQ_NOCAT,
        /* NA_SE_PL_MAGIC_WIND_NORMAL */        SEQ_NOCAT,
        /* NA_SE_PL_MAGIC_WIND_WARP */          SEQ_NOCAT,
        /* NA_SE_PL_MAGIC_SOUL_NORMAL */        SEQ_NOCAT,
        /* NA_SE_PL_ARROW_CHARGE_FIRE */        SEQ_MAGIC_CHARGE_LOOP,
        /* NA_SE_PL_ARROW_CHARGE_ICE */         SEQ_MAGIC_CHARGE_LOOP,
        /* NA_SE_PL_ARROW_CHARGE_LIGHT */       SEQ_MAGIC_CHARGE_LOOP,
        /* NA_SE_PL_DUMMY_128 */                SEQ_NOSHUFFLE,
        /* NA_SE_PL_DUMMY_129 */                SEQ_NOSHUFFLE,
        /* NA_SE_PL_DUMMY_130 */                SEQ_NOSHUFFLE,
        /* NA_SE_PL_PULL_UP_RUTO */             SEQ_NOCAT,
        /* NA_SE_PL_DUMMY_132 */                SEQ_NOSHUFFLE,
        /* NA_SE_PL_DUMMY_133 */                SEQ_NOSHUFFLE,
        /* NA_SE_PL_DUMMY_134 */                SEQ_NOSHUFFLE,
        /* NA_SE_PL_DUMMY_135 */                SEQ_NOSHUFFLE,
        /* NA_SE_PL_DUMMY_136 */                SEQ_NOSHUFFLE,
        /* NA_SE_PL_DUMMY_137 */                SEQ_NOSHUFFLE,
        /* NA_SE_PL_DUMMY_138 */                SEQ_NOSHUFFLE,
        /* NA_SE_PL_DUMMY_139 */                SEQ_NOSHUFFLE,
        /* NA_SE_PL_DUMMY_140 */                SEQ_NOSHUFFLE,
        /* NA_SE_PL_DUMMY_141 */                SEQ_NOSHUFFLE,
        /* NA_SE_PL_DUMMY_142 */                SEQ_NOSHUFFLE,
        /* NA_SE_PL_DUMMY_143 */                SEQ_NOSHUFFLE,
        /* NA_SE_PL_DUMMY_144 */                SEQ_NOSHUFFLE,
        /* NA_SE_PL_DUMMY_145 */                SEQ_NOSHUFFLE,
        /* NA_SE_PL_DUMMY_146 */                SEQ_NOSHUFFLE,
        /* NA_SE_PL_DUMMY_147 */                SEQ_NOSHUFFLE,
        /* NA_SE_PL_DUMMY_148 */                SEQ_NOSHUFFLE,
        /* NA_SE_PL_DUMMY_149 */                SEQ_NOSHUFFLE,
        /* NA_SE_PL_DUMMY_150 */                SEQ_NOSHUFFLE,
        /* NA_SE_PL_DUMMY_151 */                SEQ_NOSHUFFLE,
        /* NA_SE_PL_DUMMY_152 */                SEQ_NOSHUFFLE,
        /* NA_SE_PL_DUMMY_153 */                SEQ_NOSHUFFLE,
        /* NA_SE_PL_DUMMY_154 */                SEQ_NOSHUFFLE,
        /* NA_SE_PL_DUMMY_155 */                SEQ_NOSHUFFLE,
        /* NA_SE_PL_DUMMY_156 */                SEQ_NOSHUFFLE,
        /* NA_SE_PL_DUMMY_157 */                SEQ_NOSHUFFLE,
        /* NA_SE_PL_DUMMY_158 */                SEQ_NOSHUFFLE,
        /* NA_SE_PL_DUMMY_159 */                SEQ_NOSHUFFLE,
        /* NA_SE_PL_DUMMY_160 */                SEQ_NOSHUFFLE,
        /* NA_SE_PL_DUMMY_161 */                SEQ_NOSHUFFLE,
        /* NA_SE_PL_DUMMY_162 */                SEQ_NOSHUFFLE,
        /* NA_SE_PL_DUMMY_163 */                SEQ_NOSHUFFLE,
        /* NA_SE_PL_DUMMY_164 */                SEQ_NOSHUFFLE,
        /* NA_SE_PL_DUMMY_165 */                SEQ_NOSHUFFLE,
        /* NA_SE_PL_DUMMY_166 */                SEQ_NOSHUFFLE,
        /* NA_SE_PL_DUMMY_167 */                SEQ_NOSHUFFLE,
        /* NA_SE_PL_DUMMY_168 */                SEQ_NOSHUFFLE,
        /* NA_SE_PL_DUMMY_169 */                SEQ_NOSHUFFLE,
        /* NA_SE_PL_DUMMY_170 */                SEQ_NOSHUFFLE,
        /* NA_SE_PL_DUMMY_171 */                SEQ_NOSHUFFLE,
        /* NA_SE_PL_DUMMY_172 */                SEQ_NOSHUFFLE,
        /* NA_SE_PL_DUMMY_173 */                SEQ_NOSHUFFLE,
        /* NA_SE_PL_DUMMY_174 */                SEQ_NOSHUFFLE,
        /* NA_SE_PL_DUMMY_175 */                SEQ_NOSHUFFLE,
        /* NA_SE_PL_CRAWL */                    SEQ_CRAWL,
        /* NA_SE_PL_CRAWL_SAND */               SEQ_CRAWL,
        /* NA_SE_PL_CRAWL_CONCRETE */           SEQ_CRAWL,
        /* NA_SE_PL_CRAWL_DIRT */               SEQ_CRAWL,
        /* NA_SE_PL_CRAWL_WATER0 */             SEQ_CRAWL,
        /* NA_SE_PL_DUMMY_181 */                SEQ_CRAWL,
        /* NA_SE_PL_DUMMY_182 */                SEQ_CRAWL,
        /* NA_SE_PL_DUMMY_183 */                SEQ_CRAWL,
        /* NA_SE_PL_DUMMY_184 */                SEQ_CRAWL,
        /* NA_SE_PL_DUMMY_185 */                SEQ_CRAWL,
        /* NA_SE_PL_DUMMY_186 */                SEQ_CRAWL,
        /* NA_SE_PL_CRAWL_WOOD */               SEQ_CRAWL,
        /* NA_SE_PL_CRAWL_ICE */                SEQ_NOCAT,
        /* NA_SE_PL_DUMMY_189 */                SEQ_NOSHUFFLE,
        /* NA_SE_PL_DUMMY_190 */                SEQ_NOSHUFFLE,
        /* NA_SE_PL_DUMMY_191 */                SEQ_CRAWL,
        /* NA_SE_PL_MAGIC_SOUL_FLASH */         SEQ_MAGIC_CHARGE_LOOP,
        /* NA_SE_PL_ROLL_DUST */                SEQ_NOCAT,
        /* NA_SE_PL_DUMMY_192 */                SEQ_NOSHUFFLE,
        /* NA_SE_PL_MAGIC_SOUL_BALL */          SEQ_NOCAT,
        /* NA_SE_PL_SPIRAL_HEAL_BEAM */         SEQ_MAGIC_CHARGE_LOOP,
        /* NA_SE_PL_BOUND_NOWEAPON */           SEQ_NOCAT,
        /* NA_SE_PL_PLANT_GROW_UP */            SEQ_NOCAT,
        /* NA_SE_PL_PLANT_TALLER */             SEQ_NOCAT,
        /* NA_SE_PL_MAGIC_WIND_VANISH */        SEQ_NOCAT,
        /* NA_SE_PL_HOBBERBOOTS_LV */           SEQ_MOVE_LOOP,
        /* NA_SE_PL_PLANT_MOVE */               SEQ_MOVE_LOOP,
        /* NA_SE_EV_WALL_MOVE_SP */             SEQ_NOCAT,
        /* NA_SE_PL_DUMMY_204 */                SEQ_NOSHUFFLE,
        /* NA_SE_PL_DUMMY_205 */                SEQ_NOSHUFFLE,
        /* NA_SE_PL_DUMMY_206 */                SEQ_NOSHUFFLE,
        /* NA_SE_PL_DUMMY_207 */                SEQ_NOSHUFFLE,
        /* NA_SE_PL_SLIP_LEVEL */               SEQ_SLIP_LOOP,
        /* NA_SE_PL_SLIP_SAND_LEVEL */          SEQ_SLIP_LOOP,
        /* NA_SE_PL_SLIP_CONCRETE_LEVEL */      SEQ_SLIP_LOOP,
        /* NA_SE_PL_SLIP_DIRT_LEVEL */          SEQ_SLIP_LOOP,
        /* NA_SE_PL_SLIP_WATER0_LEVEL */        SEQ_SLIP_LOOP,
        /* NA_SE_PL_SLIP_WATER1_LEVEL */        SEQ_SLIP_LOOP,
        /* NA_SE_PL_SLIP_WATER2_LEVEL */        SEQ_SLIP_LOOP,
        /* NA_SE_PL_SLIP_MAGMA_LEVEL */         SEQ_SLIP_LOOP,
        /* NA_SE_PL_SLIP_GRASS_LEVEL */         SEQ_SLIP_LOOP,
        /* NA_SE_PL_SLIP_IRON_LEVEL */          SEQ_SLIP_LOOP,
        /* NA_SE_PL_SLIP_GLASS_LEVEL */         SEQ_SLIP_LOOP,
        /* NA_SE_PL_SLIP_WOOD_LEVEL */          SEQ_SLIP_LOOP,
        /* NA_SE_PL_DUMMY_220 */                SEQ_NOSHUFFLE,
        /* NA_SE_PL_DUMMY_221 */                SEQ_SLIP_LOOP,
        /* NA_SE_PL_SLIP_HEAVYBOOTS_LEVEL */    SEQ_SLIP_LOOP,
        /* NA_SE_PL_SLIP_ICE_LEVEL */           SEQ_SLIP_LOOP,
        /* NA_SE_PL_JUMP_METAL */               SEQ_NOCAT,
        /* NA_SE_PL_LAND_METAL */               SEQ_NOCAT,
        /* NA_SE_PL_WALK_RUNNINGMAN */          SEQ_WALK,
        /* NA_SE_PL_WALK_ZELDA_DEMO */          SEQ_NOCAT,
        /* NA_SE_PL_YOBI_DATA02 */              SEQ_NOSHUFFLE,
        /* NA_SE_PL_YOBI_DATA03 */              SEQ_NOSHUFFLE,
        /* NA_SE_PL_YOBI_DATA04 */              SEQ_NOSHUFFLE,
        /* NA_SE_PL_YOBI_DATA05 */              SEQ_NOSHUFFLE,
        /* NA_SE_PL_YOBI_DATA06 */              SEQ_NOSHUFFLE,
        /* NA_SE_PL_YOBI_DATA07 */              SEQ_NOSHUFFLE,
        /* NA_SE_PL_YOBI_DATA08 */              SEQ_NOSHUFFLE,
        /* NA_SE_PL_YOBI_DATA09 */              SEQ_NOSHUFFLE,
        /* NA_SE_PL_YOBI_DATA10 */              SEQ_NOSHUFFLE,
        /* NA_SE_PL_YOBI_DATA11 */              SEQ_NOSHUFFLE,
        /* NA_SE_PL_YOBI_DATA12 */              SEQ_NOSHUFFLE,
        /* NA_SE_PL_YOBI_DATA13 */              SEQ_NOSHUFFLE,
        /* NA_SE_PL_YOBI_DATA14 */              SEQ_NOSHUFFLE,
        /* NA_SE_IT_SWORD_IMPACT */             SEQ_NOCAT,
        /* NA_SE_IT_SWORD_SWING */              SEQ_WEAPON_SWING,
        /* NA_SE_IT_SWORD_PUTAWAY */            SEQ_NOCAT,
        /* NA_SE_IT_SWORD_PICKOUT */            SEQ_NOCAT,
        /* NA_SE_IT_ARROW_SHOT */               SEQ_PROJECTILE_SHOT,
        /* NA_SE_IT_BOOMERANG_THROW */          SEQ_NOCAT,
        /* NA_SE_IT_SHIELD_BOUND */             SEQ_WEAPON_HIT,
        /* NA_SE_IT_BOW_DRAW */                 SEQ_NOCAT,
        /* NA_SE_IT_SHIELD_REFLECT_SW */        SEQ_WEAPON_HIT,
        /* NA_SE_IT_ARROW_STICK_HRAD */         SEQ_NOCAT,
        /* NA_SE_IT_HAMMER_HIT */               SEQ_WEAPON_HIT,
        /* NA_SE_IT_HOOKSHOT_CHAIN */           SEQ_NOCAT,
        /* NA_SE_IT_SHIELD_REFLECT_MG */        SEQ_NOCAT,
        /* NA_SE_IT_BOMB_IGNIT */               SEQ_NOCAT,
        /* NA_SE_IT_BOMB_EXPLOSION */           SEQ_EXPLOSION,
        /* NA_SE_IT_BOMB_UNEXPLOSION */         SEQ_NOCAT,
        /* NA_SE_IT_BOOMERANG_FLY */            SEQ_THROW_LOOP,
        /* NA_SE_IT_SWORD_STRIKE */             SEQ_NOCAT,
        /* NA_SE_IT_HAMMER_SWING */             SEQ_WEAPON_SWING,
        /* NA_SE_IT_HOOKSHOT_REFLECT */         SEQ_WEAPON_HIT,
        /* NA_SE_IT_ARROW_STICK_CRE */          SEQ_NOCAT,
        /* NA_SE_IT_ARROW_STICK_OBJ */          SEQ_NOCAT,
        /* NA_SE_IT_DUMMY */                    SEQ_NOSHUFFLE,
        /* NA_SE_IT_DUMMY2 */                   SEQ_NOSHUFFLE,
        /* NA_SE_IT_SWORD_SWING_HARD */         SEQ_WEAPON_SWING,
        /* NA_SE_IT_DUMMY3 */                   SEQ_NOSHUFFLE,
        /* NA_SE_IT_WALL_HIT_HARD */            SEQ_WEAPON_HIT,
        /* NA_SE_IT_WALL_HIT_SOFT */            SEQ_WEAPON_HIT,
        /* NA_SE_IT_STONE_HIT */                SEQ_NOCAT,
        /* NA_SE_IT_WOODSTICK_BROKEN */         SEQ_NOCAT,
        /* NA_SE_IT_LASH */                     SEQ_WEAPON_SWING,
        /* NA_SE_IT_SHIELD_POSTURE */           SEQ_NOCAT,
        /* NA_SE_IT_SLING_SHOT */               SEQ_PROJECTILE_SHOT,
        /* NA_SE_IT_SLING_DRAW */               SEQ_NOCAT,
        /* NA_SE_IT_SWORD_CHARGE */             SEQ_MAGIC_CHARGE_LOOP,
        /* NA_SE_IT_ROLLING_CUT */              SEQ_WEAPON_SWING,
        /* NA_SE_IT_SWORD_STRIKE_HARD */        SEQ_NOCAT,
        /* NA_SE_IT_SLING_REFLECT */            SEQ_WEAPON_HIT,
        /* NA_SE_IT_SHIELD_REMOVE */            SEQ_NOCAT,
        /* NA_SE_IT_HOOKSHOT_READY */           SEQ_NOCAT,
        /* NA_SE_IT_HOOKSHOT_RECEIVE */         SEQ_NOCAT,
        /* NA_SE_IT_HOOKSHOT_STICK_OBJ */       SEQ_WEAPON_HIT,
        /* NA_SE_IT_SWORD_REFLECT_MG */         SEQ_NOCAT,
        /* NA_SE_IT_DEKU */                     SEQ_WEAPON_HIT,
        /* NA_SE_IT_WALL_HIT_BUYO */            SEQ_NOCAT,
        /* NA_SE_IT_SWORD_PUTAWAY_STN */        SEQ_NOCAT,
        /* NA_SE_IT_ROLLING_CUT_LV1 */          SEQ_NOCAT,
        /* NA_SE_IT_ROLLING_CUT_LV2 */          SEQ_NOCAT,
        /* NA_SE_IT_BOW_FLICK */                SEQ_NOCAT,
        /* NA_SE_IT_BOMBCHU_MOVE */             SEQ_MOVE_LOOP,
        /* NA_SE_IT_SHIELD_CHARGE_LV1 */        SEQ_MAGIC_CHARGE_LOOP,
        /* NA_SE_IT_SHIELD_CHARGE_LV2 */        SEQ_MAGIC_CHARGE_LOOP,
        /* NA_SE_IT_SHIELD_CHARGE_LV3 */        SEQ_MAGIC_CHARGE_LOOP,
        /* NA_SE_IT_SLING_FLICK */              SEQ_NOCAT,
        /* NA_SE_IT_SWORD_STICK_STN */          SEQ_NOCAT,
        /* NA_SE_IT_REFLECTION_WOOD */          SEQ_WEAPON_HIT,
        /* NA_SE_IT_SHIELD_REFLECT_MG2 */       SEQ_NOCAT,
        /* NA_SE_IT_MAGIC_ARROW_SHOT */         SEQ_PROJECTILE_SHOT,
        /* NA_SE_IT_EXPLOSION_FRAME */          SEQ_EXPLOSION,
        /* NA_SE_IT_EXPLOSION_ICE */            SEQ_EXPLOSION,
        /* NA_SE_IT_EXPLOSION_LIGHT */          SEQ_EXPLOSION,
        /* NA_SE_IT_FISHING_REEL_SLOW */        SEQ_NOCAT,
        /* NA_SE_IT_FISHING_REEL_HIGH */        SEQ_NOCAT,
        /* NA_SE_IT_PULL_FISHING_ROD */         SEQ_NOCAT,
        /* NA_SE_IT_DM_FLYING_GOD_PASS */       SEQ_NOCAT,
        /* NA_SE_IT_DM_FLYING_GOD_DASH */       SEQ_NOCAT,
        /* NA_SE_IT_DM_RING_EXPLOSION */        SEQ_EXPLOSION,
        /* NA_SE_IT_DM_RING_GATHER */           SEQ_NOCAT,
        /* NA_SE_IT_INGO_HORSE_NEIGH */         SEQ_NOCAT, // Blank
        /* NA_SE_IT_EARTHQUAKE */               SEQ_EXPLOSION,
        /* NA_SE_IT_DUMMY4 */                   SEQ_NOSHUFFLE,
        /* NA_SE_IT_KAKASHI_JUMP */             SEQ_JUMP,
        /* NA_SE_IT_FLAME */                    SEQ_NOCAT,
        /* NA_SE_IT_SHIELD_BEAM */              SEQ_NOCAT,
        /* NA_SE_IT_FISHING_HIT */              SEQ_NOCAT,
        /* NA_SE_IT_GOODS_APPEAR */             SEQ_NOCAT,
        /* NA_SE_IT_MAJIN_SWORD_BROKEN */       SEQ_WEAPON_HIT,
        /* NA_SE_IT_HAND_CLAP */                SEQ_WEAPON_HIT,
        /* NA_SE_IT_MASTER_SWORD_SWING */       SEQ_WEAPON_SWING,
        /* NA_SE_IT_DUMMY5 */                   SEQ_NOSHUFFLE,
        /* NA_SE_IT_YOBI19 */                   SEQ_NOSHUFFLE,
        /* NA_SE_FISHING_REEL_SLOW2 */          SEQ_NOCAT,
        /* NA_SE_IT_SPIDERNET_HIT1 */           SEQ_WEAPON_HIT,
        /* NA_SE_IT_LURE_LAND1 */               SEQ_NOCAT,
        /* NA_SE_IT_HOOKSHOT_STICK_OBJ_WATER */ SEQ_WEAPON_HIT,
        /* NA_SE_IT_SWORD_PICKOUT_GANON */      SEQ_NOCAT,
        /* NA_SE_IT_BOMB_IGNIT_DODO_M */        SEQ_NOCAT,
        /* NA_SE_IT_YOBI05 */                   SEQ_NOSHUFFLE,
        /* NA_SE_IT_YOBI06 */                   SEQ_NOSHUFFLE,
        /* NA_SE_IT_YOBI07 */                   SEQ_NOSHUFFLE,
        /* NA_SE_IT_YOBI08 */                   SEQ_NOSHUFFLE,
        /* NA_SE_IT_YOBI09 */                   SEQ_NOSHUFFLE,
        /* NA_SE_IT_YOBI10 */                   SEQ_NOSHUFFLE,
        /* NA_SE_IT_YOBI11 */                   SEQ_NOSHUFFLE,
        /* NA_SE_IT_YOBI12 */                   SEQ_NOSHUFFLE,
        /* NA_SE_IT_YOBI13 */                   SEQ_NOSHUFFLE,
        /* NA_SE_IT_YOBI14 */                   SEQ_NOSHUFFLE,
        /* NA_SE_IT_YOBI15 */                   SEQ_NOSHUFFLE,
        /* NA_SE_IT_YOBI16 */                   SEQ_NOSHUFFLE,
        /* NA_SE_IT_YOBI17 */                   SEQ_NOSHUFFLE,
        /* NA_SE_IT_YOBI18 */                   SEQ_NOSHUFFLE,
        /* NA_SE_EV_DOOR_OPEN */                SEQ_DOOR_OPEN,
        /* NA_SE_EV_DOOR_CLOSE */               SEQ_DOOR_CLOSE,
        /* NA_SE_EV_EXPLOSION */                SEQ_EXPLOSION,
        /* NA_SE_EV_HORSE_WALK */               SEQ_WALK,
        /* NA_SE_EV_HORSE_RUN */                SEQ_WALK,
        /* NA_SE_EV_HORSE_NEIGH */              SEQ_MONSTER_CRY_MEDIUM,
        /* NA_SE_EV_RIVER_STREAM */             SEQ_AMBIENCE,
        /* NA_SE_EV_WATER_WALL_BIG */           SEQ_AMBIENCE,
        /* NA_SE_EV_OUT_OF_WATER */             SEQ_NOCAT,
        /* NA_SE_EV_DIVE_WATER */               SEQ_NOCAT,
        /* NA_SE_EV_ROCK_SLIDE */               SEQ_NOCAT,
        /* NA_SE_EV_MAGMA_LEVEL */              SEQ_AMBIENCE,
        /* NA_SE_EV_BRIDGE_OPEN */              SEQ_NOCAT,
        /* NA_SE_EV_BRIDGE_CLOSE */             SEQ_NOCAT,
        /* NA_SE_EV_BRIDGE_OPEN_STOP */         SEQ_NOCAT,
        /* NA_SE_EV_BRIDGE_CLOSE_STOP */        SEQ_NOCAT,
        /* NA_SE_EV_WALL_BROKEN */              SEQ_NOCAT,
        /* NA_SE_EV_CHICKEN_CRY_N */            SEQ_MONSTER_CRY_MEDIUM,
        /* NA_SE_EV_CHICKEN_CRY_A */            SEQ_MONSTER_CRY_SHORT,
        /* NA_SE_EV_CHICKEN_CRY_M */            SEQ_MONSTER_CRY_LONG,
        /* NA_SE_EV_SLIDE_DOOR_OPEN */          SEQ_DOOR_OPEN,
        /* NA_SE_EV_FOOT_SWITCH */              SEQ_NOCAT,
        /* NA_SE_EV_HORSE_GROAN */              SEQ_MONSTER_CRY_SHORT,
        /* NA_SE_EV_BOMB_DROP_WATER */          SEQ_NOCAT,
        /* NA_SE_EV_HORSE_JUMP */               SEQ_JUMP,
        /* NA_SE_EV_HORSE_LAND */               SEQ_LAND,
        /* NA_SE_EV_HORSE_SLIP */               SEQ_SLIP,
        /* NA_SE_EV_FAIRY_DASH */               SEQ_NOCAT,
        /* NA_SE_EV_SLIDE_DOOR_CLOSE */         SEQ_DOOR_CLOSE,
        /* NA_SE_EV_STONE_BOUND */              SEQ_NOCAT,
        /* NA_SE_EV_STONE_STATUE_OPEN */        SEQ_NOCAT,
        /* NA_SE_EV_TBOX_UNLOCK */              SEQ_NOCAT,
        /* NA_SE_EV_TBOX_OPEN */                SEQ_NOCAT,
        /* NA_SE_SY_TIMER */                    SEQ_NOSHUFFLE,
        /* NA_SE_EV_FLAME_IGNITION */           SEQ_NOCAT,
        /* NA_SE_EV_SPEAR_HIT */                SEQ_WEAPON_HIT,
        /* NA_SE_EV_ELEVATOR_MOVE */            SEQ_NOCAT,
        /* NA_SE_EV_WARP_HOLE */                SEQ_NOCAT,
        /* NA_SE_EV_LINK_WARP */                SEQ_NOCAT,
        /* NA_SE_EV_PILLAR_SINK */              SEQ_NOCAT,
        /* NA_SE_EV_WATER_WALL */               SEQ_AMBIENCE,
        /* NA_SE_EV_RIVER_STREAM_S */           SEQ_AMBIENCE,
        /* NA_SE_EV_RIVER_STREAM_F */           SEQ_AMBIENCE,
        /* NA_SE_EV_HORSE_LAND2 */              SEQ_LAND,
        /* NA_SE_EV_HORSE_SANDDUST */           SEQ_NOCAT,
        /* NA_SE_EV_DUMMY45 */                  SEQ_NOSHUFFLE,
        /* NA_SE_EV_LIGHTNING */                SEQ_EXPLOSION,
        /* NA_SE_EV_BOMB_BOUND */               SEQ_NOCAT,
        /* NA_SE_EV_WATERDROP */                SEQ_AMBIENCE,
        /* NA_SE_EV_TORCH */                    SEQ_NOCAT,
        /* NA_SE_EV_MAGMA_LEVEL_M */            SEQ_AMBIENCE,
        /* NA_SE_EV_FIRE_PILLAR */              SEQ_NOCAT,
        /* NA_SE_EV_FIRE_PLATE */               SEQ_NOCAT,
        /* NA_SE_EV_BLOCK_BOUND */              SEQ_NOCAT,
        /* NA_SE_EV_METALDOOR_SLIDE */          SEQ_NOCAT,
        /* NA_SE_EV_METALDOOR_STOP */           SEQ_NOCAT,
        /* NA_SE_EV_BLOCK_SHAKE */              SEQ_NOCAT,
        /* NA_SE_EV_BOX_BREAK */                SEQ_NOCAT,
        /* NA_SE_EV_HAMMER_SWITCH */            SEQ_NOCAT,
        /* NA_SE_EV_MAGMA_LEVEL_L */            SEQ_AMBIENCE,
        /* NA_SE_EV_SPEAR_FENCE */              SEQ_NOCAT,
        /* NA_SE_EV_GANON_HORSE_NEIGH */        SEQ_MONSTER_CRY_MEDIUM,
        /* NA_SE_EV_GANON_HORSE_GROAN */        SEQ_MONSTER_CRY_SHORT,
        /* NA_SE_EV_FANTOM_WARP_S */            SEQ_NOCAT,
        /* NA_SE_EV_FANTOM_WARP_L */            SEQ_NOCAT,
        /* NA_SE_EV_FOUNTAIN */                 SEQ_AMBIENCE,
        /* NA_SE_EV_KID_HORSE_WALK */           SEQ_WALK,
        /* NA_SE_EV_KID_HORSE_RUN */            SEQ_WALK,
        /* NA_SE_EV_KID_HORSE_NEIGH */          SEQ_MONSTER_CRY_MEDIUM,
        /* NA_SE_EV_KID_HORSE_GROAN */          SEQ_MONSTER_CRY_SHORT,
        /* NA_SE_EV_WHITE_OUT */                SEQ_NOCAT,
        /* NA_SE_EV_LIGHT_GATHER */             SEQ_NOCAT,
        /* NA_SE_EV_TREE_CUT */                 SEQ_NOCAT,
        /* NA_SE_EV_VOLCANO */                  SEQ_NOCAT,
        /* NA_SE_EV_GUILLOTINE_UP */            SEQ_NOCAT,
        /* NA_SE_EV_GUILLOTINE_BOUND */         SEQ_NOCAT,
        /* NA_SE_EV_ROLLCUTTER_MOTOR */         SEQ_NOCAT,
        /* NA_SE_EV_CHINETRAP_DOWN */           SEQ_NOCAT,
        /* NA_SE_EV_PLANT_BROKEN */             SEQ_NOCAT,
        /* NA_SE_EV_SHIP_BELL */                SEQ_NOCAT,
        /* NA_SE_EV_FLUTTER_FLAG */             SEQ_AMBIENCE,
        /* NA_SE_EV_TRAP_BOUND */               SEQ_NOCAT,
        /* NA_SE_EV_ROCK_BROKEN */              SEQ_NOCAT,
        /* NA_SE_EV_FANTOM_WARP_S2 */           SEQ_NOCAT,
        /* NA_SE_EV_FANTOM_WARP_L2 */           SEQ_NOCAT,
        /* NA_SE_EV_COFFIN_CAP_OPEN */          SEQ_NOCAT,
        /* NA_SE_EV_COFFIN_CAP_BOUND */         SEQ_NOCAT,
        /* NA_SE_EV_WIND_TRAP */                SEQ_NOCAT,
        /* NA_SE_EV_TRAP_OBJ_SLIDE */           SEQ_NOCAT,
        /* NA_SE_EV_METALDOOR_OPEN */           SEQ_DOOR_OPEN,
        /* NA_SE_EV_METALDOOR_CLOSE */          SEQ_DOOR_CLOSE,
        /* NA_SE_EV_BURN_OUT */                 SEQ_NOCAT,
        /* NA_SE_EV_BLOCKSINK */                SEQ_NOCAT,
        /* NA_SE_EV_CROWD */                    SEQ_AMBIENCE,
        /* NA_SE_EV_WATER_LEVEL_DOWN */         SEQ_NOCAT,
        /* NA_SE_EV_NAVY_VANISH */              SEQ_NOCAT,
        /* NA_SE_EV_LADDER_DOUND */             SEQ_NOCAT,
        /* NA_SE_EV_WEB_VIBRATION */            SEQ_NOCAT,
        /* NA_SE_EV_WEB_BROKEN */               SEQ_NOCAT,
        /* NA_SE_EV_ROLL_STAND */               SEQ_NOCAT,
        /* NA_SE_EV_BUYODOOR_OPEN */            SEQ_DOOR_OPEN,
        /* NA_SE_EV_BUYODOOR_CLOSE */           SEQ_DOOR_CLOSE,
        /* NA_SE_EV_WOODDOOR_OPEN */            SEQ_NOCAT,
        /* NA_SE_EV_METALGATE_OPEN */           SEQ_NOCAT,
        /* NA_SE_IT_SCOOP_UP_WATER */           SEQ_NOCAT,
        /* NA_SE_EV_FISH_LEAP */                SEQ_NOCAT,
        /* NA_SE_EV_KAKASHI_SWING */            SEQ_NOCAT,
        /* NA_SE_EV_KAKASHI_ROLL */             SEQ_NOCAT,
        /* NA_SE_EV_BOTTLE_CAP_OPEN */          SEQ_NOCAT,
        /* NA_SE_EV_JABJAB_BREATHE */           SEQ_NOCAT,
        /* NA_SE_EV_SPIRIT_STONE */             SEQ_NOCAT,
        /* NA_SE_EV_TRIFORCE_FLASH */           SEQ_NOCAT,
        /* NA_SE_EV_FALL_DOWN_DIRT */           SEQ_NOCAT,
        /* NA_SE_EV_NAVY_FLY */                 SEQ_NOCAT,
        /* NA_SE_EV_NAVY_CRASH */               SEQ_NOCAT,
        /* NA_SE_EV_WOOD_HIT */                 SEQ_NOCAT,
        /* NA_SE_EV_SCOOPUP_WATER */            SEQ_AMBIENCE, // Waterfall + bubbling
        /* NA_SE_EV_DROP_FALL */                SEQ_NOCAT,
        /* NA_SE_EV_WOOD_GEAR */                SEQ_NOCAT,
        /* NA_SE_EV_TREE_SWING */               SEQ_NOCAT,
        /* NA_SE_EV_HORSE_RUN_LEVEL */          SEQ_NOCAT,
        /* NA_SE_EV_ELEVATOR_MOVE2 */           SEQ_NOCAT,
        /* NA_SE_EV_ELEVATOR_STOP */            SEQ_NOCAT,
        /* NA_SE_EV_TRE_BOX_APPEAR */           SEQ_NOCAT,
        /* NA_SE_EV_CHAIN_KEY_UNLOCK */         SEQ_NOCAT,
        /* NA_SE_EV_SPINE_TRAP_MOVE */          SEQ_NOCAT,
        /* NA_SE_EV_HEALING */                  SEQ_MAGIC_CHARGE_LOOP,
        /* NA_SE_EV_GREAT_FAIRY_APPEAR */       SEQ_NOCAT,
        /* NA_SE_EV_GREAT_FAIRY_VANISH */       SEQ_NOCAT,
        /* NA_SE_EV_RED_EYE */                  SEQ_NOCAT,
        /* NA_SE_EV_ROLL_STAND_2 */             SEQ_NOCAT,
        /* NA_SE_EV_WALL_SLIDE */               SEQ_NOCAT,
        /* NA_SE_EV_TRE_BOX_FLASH */            SEQ_NOCAT,
        /* NA_SE_EV_WINDMILL_LEVEL */           SEQ_AMBIENCE,
        /* NA_SE_EV_GOTO_HEAVEN */              SEQ_NOCAT,
        /* NA_SE_EV_POT_BROKEN */               SEQ_NOCAT,
        /* NA_SE_PL_PUT_DOWN_POT */             SEQ_NOCAT,
        /* NA_SE_EV_DIVE_INTO_WATER */          SEQ_NOCAT,
        /* NA_SE_EV_JUMP_OUT_WATER */           SEQ_NOCAT,
        /* NA_SE_EV_GOD_FLYING */               SEQ_NOCAT,
        /* NA_SE_EV_TRIFORCE */                 SEQ_NOCAT,
        /* NA_SE_EV_AURORA */                   SEQ_NOCAT,
        /* NA_SE_EV_DEKU_DEATH */               SEQ_NOCAT,
        /* NA_SE_EV_BUYOSTAND_RISING */         SEQ_NOCAT,
        /* NA_SE_EV_BUYOSTAND_FALL */           SEQ_NOCAT,
        /* NA_SE_EV_BUYOSHUTTER_OPEN */         SEQ_NOCAT,
        /* NA_SE_EV_BUYOSHUTTER_CLOSE */        SEQ_NOCAT,
        /* NA_SE_EV_STONEDOOR_STOP */           SEQ_NOCAT,
        /* NA_SE_EV_S_STONE_REVIVAL */          SEQ_NOCAT,
        /* NA_SE_EV_MEDAL_APPEAR_S */           SEQ_NOCAT,
        /* NA_SE_EV_HUMAN_BOUND */              SEQ_NOCAT,
        /* NA_SE_EV_MEDAL_APPEAR_L */           SEQ_NOCAT,
        /* NA_SE_EV_EARTHQUAKE */               SEQ_NOCAT,
        /* NA_SE_EV_SHUT_BY_CRYSTAL */          SEQ_NOCAT,
        /* NA_SE_EV_GOD_LIGHTBALL_2 */          SEQ_NOCAT,
        /* NA_SE_EV_RUN_AROUND */               SEQ_NOCAT,
        /* NA_SE_EV_CONSENTRATION */            SEQ_NOCAT,
        /* NA_SE_EV_TIMETRIP_LIGHT */           SEQ_NOCAT,
        /* NA_SE_EV_BUYOSTAND_STOP_A */         SEQ_NOCAT,
        /* NA_SE_EV_BUYOSTAND_STOP_U */         SEQ_NOCAT,
        /* NA_SE_EV_OBJECT_FALL */              SEQ_NOCAT,
        /* NA_SE_EV_JUMP_CONC */                SEQ_NOCAT,
        /* NA_SE_EV_ICE_MELT */                 SEQ_NOCAT,
        /* NA_SE_EV_FIRE_PILLAR_S */            SEQ_NOCAT,
        /* NA_SE_EV_BLOCK_RISING */             SEQ_NOCAT,
        /* NA_SE_EV_NABALL_VANISH */            SEQ_NOCAT,
        /* NA_SE_EV_SARIA_MELODY */             SEQ_NOCAT,
        /* NA_SE_EV_LINK_WARP_OUT */            SEQ_NOCAT,
        /* NA_SE_EV_FIATY_HEAL */               SEQ_NOCAT,
        /* NA_SE_EV_CHAIN_KEY_UNLOCK_B */       SEQ_NOCAT,
        /* NA_SE_EV_WOODBOX_BREAK */            SEQ_NOCAT,
        /* NA_SE_EV_PUT_DOWN_WOODBOX */         SEQ_NOCAT,
        /* NA_SE_EV_LAND_DIRT */                SEQ_NOCAT,
        /* NA_SE_EV_FLOOR_ROLLING */            SEQ_NOCAT,
        /* NA_SE_EV_DOG_CRY_EVENING */          SEQ_MONSTER_CRY_LONG,
        /* NA_SE_EV_JABJAB_HICCUP */            SEQ_NOCAT,
        /* NA_SE_EV_NALE_MAGIC */               SEQ_NOCAT,
        /* NA_SE_EV_FROG_JUMP */                SEQ_JUMP,
        /* NA_SE_EV_ICE_FREEZE */               SEQ_NOCAT,
        /* NA_SE_EV_BURNING */                  SEQ_NOCAT,
        /* NA_SE_EV_WOODPLATE_BOUND */          SEQ_NOCAT,
        /* NA_SE_EV_GORON_WATER_DROP */         SEQ_NOCAT,
        /* NA_SE_EV_JABJAB_GROAN */             SEQ_MONSTER_CRY_LONG,
        /* NA_SE_EV_DARUMA_VANISH */            SEQ_NOCAT,
        /* NA_SE_EV_BIGBALL_ROLL */             SEQ_NOCAT,
        /* NA_SE_EV_ELEVATOR_MOVE3 */           SEQ_NOCAT,
        /* NA_SE_EV_DIAMOND_SWITCH */           SEQ_NOCAT,
        /* NA_SE_EV_FLAME_OF_FIRE */            SEQ_NOCAT,
        /* NA_SE_EV_RAINBOW_SHOWER */           SEQ_NOCAT,
        /* NA_SE_EV_FLYING_AIR */               SEQ_NOCAT,
        /* NA_SE_EV_PASS_AIR */                 SEQ_NOCAT,
        /* NA_SE_EV_COME_UP_DEKU_JR */          SEQ_NOCAT,
        /* NA_SE_EV_SAND_STORM */               SEQ_NOCAT,
        /* NA_SE_EV_TRIFORCE_MARK */            SEQ_NOCAT,
        /* NA_SE_EV_GRAVE_EXPLOSION */          SEQ_NOCAT,
        /* NA_SE_EV_LURE_MOVE_W */              SEQ_NOCAT,
        /* NA_SE_EV_POT_MOVE_START */           SEQ_NOCAT,
        /* NA_SE_EV_DIVE_INTO_WATER_L */        SEQ_NOCAT,
        /* NA_SE_EV_OUT_OF_WATER_L */           SEQ_NOCAT,
        /* NA_SE_EV_GANON_MANTLE */             SEQ_NOCAT,
        /* NA_SE_EV_DIG_UP */                   SEQ_NOCAT,
        /* NA_SE_EV_WOOD_BOUND */               SEQ_NOCAT,
        /* NA_SE_EV_WATER_BUBBLE */             SEQ_NOCAT,
        /* NA_SE_EV_ICE_BROKEN */               SEQ_NOCAT,
        /* NA_SE_EV_FROG_GROW_UP */             SEQ_NOCAT,
        /* NA_SE_EV_WATER_CONVECTION */         SEQ_NOCAT,
        /* NA_SE_EV_GROUND_GATE_OPEN */         SEQ_NOCAT,
        /* NA_SE_EV_FACE_BREAKDOWN */           SEQ_NOCAT,
        /* NA_SE_EV_FACE_EXPLOSION */           SEQ_NOCAT,
        /* NA_SE_EV_FACE_CRUMBLE_SLOW */        SEQ_NOCAT,
        /* NA_SE_EV_ROUND_TRAP_MOVE */          SEQ_NOCAT,
        /* NA_SE_EV_HIT_SOUND */                SEQ_NOCAT,
        /* NA_SE_EV_ICE_SWING */                SEQ_NOCAT,
        /* NA_SE_EV_DOWN_TO_GROUND */           SEQ_NOCAT,
        /* NA_SE_EV_KENJA_ENVIROMENT_0 */       SEQ_AMBIENCE,
        /* NA_SE_EV_KENJA_ENVIROMENT_1 */       SEQ_NOCAT,
        /* NA_SE_EV_SMALL_DOG_BARK */           SEQ_MONSTER_CRY_SHORT,
        /* NA_SE_EV_ZELDA_POWER */              SEQ_NOCAT,
        /* NA_SE_EV_RAIN */                     SEQ_AMBIENCE,
        /* NA_SE_EV_IRON_DOOR_OPEN */           SEQ_DOOR_OPEN,
        /* NA_SE_EV_IRON_DOOR_CLOSE */          SEQ_DOOR_CLOSE,
        /* NA_SE_EV_WHIRLPOOL */                SEQ_NOCAT,
        /* NA_SE_EV_TOWER_PARTS_BROKEN */       SEQ_NOCAT,
        /* NA_SE_EV_COW_CRY */                  SEQ_MONSTER_CRY_MEDIUM,
        /* NA_SE_EV_METAL_BOX_BOUND */          SEQ_NOCAT,
        /* NA_SE_EV_ELECTRIC_EXPLOSION */       SEQ_NOCAT,
        /* NA_SE_EV_HEAVY_THROW */              SEQ_NOCAT,
        /* NA_SE_EV_FROG_CRY_0 */               SEQ_MONSTER_CRY_SHORT,
        /* NA_SE_EV_FROG_CRY_1 */               SEQ_MONSTER_CRY_SHORT,
        /* NA_SE_EV_COW_CRY_LV */               SEQ_NOCAT,
        /* NA_SE_EV_RONRON_DOOR_CLOSE */        SEQ_NOCAT,
        /* NA_SE_EV_BUTTERFRY_TO_FAIRY */       SEQ_NOCAT,
        /* NA_SE_EV_FIVE_COUNT_LUPY */          SEQ_NOCAT,
        /* NA_SE_EV_STONE_GROW_UP */            SEQ_NOCAT,
        /* NA_SE_EV_STONE_LAUNCH */             SEQ_NOCAT,
        /* NA_SE_EV_STONE_ROLLING */            SEQ_NOCAT,
        /* NA_SE_EV_TOGE_STICK_ROLLING */       SEQ_NOCAT,
        /* NA_SE_EV_TOWER_ENERGY */             SEQ_MAGIC_CHARGE_LOOP,
        /* NA_SE_EV_TOWER_BARRIER */            SEQ_MAGIC_CHARGE_LOOP,
        /* NA_SE_EV_CHIBI_WALK */               SEQ_WALK,
        /* NA_SE_EV_KNIGHT_WALK */              SEQ_WALK,
        /* NA_SE_EV_PILLAR_MOVE_STOP */         SEQ_NOCAT,
        /* NA_SE_EV_ERUPTION_CLOUD */           SEQ_NOCAT,
        /* NA_SE_EV_LINK_WARP_OUT_LV */         SEQ_NOCAT,
        /* NA_SE_EV_LINK_WARP_IN */             SEQ_NOCAT,
        /* NA_SE_EV_OCARINA_BMELO_0 */          SEQ_NOCAT,
        /* NA_SE_EV_OCARINA_BMELO_1 */          SEQ_NOCAT,
        /* NA_SE_EV_EXPLOSION_FOR_RENZOKU */    SEQ_NOCAT,
        /* NA_SE_EV_ELEVATOR_MOVE_KABE1 */      SEQ_NOCAT,
        /* NA_SE_EV_RIVER_STREAM_F_IDO */       SEQ_AMBIENCE,
        /* NA_SE_EV_GUILLOTINE_BOUND */         SEQ_NOCAT,
        /* NA_SE_EV_HEALING_TOU */              SEQ_NOCAT,
        /* NA_SE_EV_RUMUBLE_KEMURI */           SEQ_NOCAT,
        /* NA_SE_EV_GANON_HADOU */              SEQ_NOCAT,
        /* NA_SE_EV_KANOKE_OPEN */              SEQ_NOCAT,
        /* NA_SE_EV_KANOKE_CLOSE */             SEQ_NOCAT,
        /* NA_SE_EV_SEEK_CLOTH1 */              SEQ_NOCAT,
        /* NA_SE_EV_SEEK_CLOTH2 */              SEQ_NOCAT,
        /* NA_SE_EV_BOTTLE_CAP_CLOSE */         SEQ_NOCAT,
        /* NA_SE_EV_ELEVATOR_MOVE_KABE2 */      SEQ_NOCAT,
        /* NA_SE_EV_WATER_LEVEL_DOWN_STOP */    SEQ_NOCAT,
        /* NA_SE_EV_DAIKU_CLOTH1 */             SEQ_NOCAT,
        /* NA_SE_EV_DEMO_EPONA_LAND */          SEQ_NOCAT,
        /* NA_SE_EV_DIVE_INTO_WATER_BLOCK */    SEQ_NOCAT,
        /* NA_SE_EV_TORCH2 */                   SEQ_NOCAT,
        /* NA_SE_EV_TORCH3 */                   SEQ_NOCAT,
        /* NA_SE_EV_TORCH4 */                   SEQ_NOCAT,
        /* NA_SE_EV_TORCH5 */                   SEQ_NOCAT,
        /* NA_SE_EV_EARTHQUAKE_LAST */          SEQ_NOCAT,
        /* NA_SE_EV_YAMI_TRAP_CHAIN */          SEQ_NOCAT,
        /* NA_SE_EV_FLAME_IGNITION_GANON */     SEQ_NOCAT,
        /* NA_SE_EV_MGANON_DOWN2 */             SEQ_NOCAT,
        /* NA_SE_EV_EXPLOSION_HOUKAI */         SEQ_NOCAT,
        /* NA_SE_EV_BLOCKSINK_GANON */          SEQ_NOCAT,
        /* NA_SE_EV_DOG_WALK */                 SEQ_WALK,
        /* NA_SE_EV_GANON_HOUKAI_KEMURI1 */     SEQ_NOCAT,
        /* NA_SE_EV_YOBI21 */                   SEQ_NOSHUFFLE,
        /* NA_SE_EV_YOBI22 */                   SEQ_NOSHUFFLE,
        /* NA_SE_EV_YOBI23 */                   SEQ_NOSHUFFLE,
        /* NA_SE_EV_YOBI24 */                   SEQ_NOSHUFFLE,
        /* NA_SE_EV_YOBI25 */                   SEQ_NOSHUFFLE,
        /* NA_SE_EN_DODO_J_WALK */              SEQ_WALK_LOUD,
        /* NA_SE_EN_DODO_J_CRY */               SEQ_MONSTER_CRY_MEDIUM,
        /* NA_SE_EN_DODO_J_FIRE */              SEQ_NOCAT,
        /* NA_SE_EN_DODO_J_DAMAGE */            SEQ_MONSTER_DAMAGED,
        /* NA_SE_EN_DODO_J_DEAD */              SEQ_MONSTER_DEAD,
        /* NA_SE_EN_DODO_K_CRY */               SEQ_MONSTER_CRY_MEDIUM,
        /* NA_SE_EN_DODO_K_DAMAGE */            SEQ_MONSTER_DAMAGED,
        /* NA_SE_EN_DODO_K_DEAD */              SEQ_MONSTER_DEAD,
        /* NA_SE_EN_DODO_K_WALK */              SEQ_WALK_LOUD,
        /* NA_SE_EN_DODO_K_FIRE */              SEQ_NOCAT,
        /* NA_SE_EN_GOMA_WALK */                SEQ_WALK_LOUD,
        /* NA_SE_EN_GOMA_HIGH */                SEQ_NOCAT,
        /* NA_SE_EN_GOMA_CLIM */                SEQ_NOCAT,
        /* NA_SE_EN_GOMA_DOWN */                SEQ_NOCAT,
        /* NA_SE_EN_GOMA_CRY1 */                SEQ_MONSTER_CRY_MEDIUM,
        /* NA_SE_EN_GOMA_CRY2 */                SEQ_NOCAT,
        /* NA_SE_EN_GOMA_DAM1 */                SEQ_NOCAT,
        /* NA_SE_EN_GOMA_DAM2 */                SEQ_NOCAT,
        /* NA_SE_EN_GOMA_DEAD */                SEQ_MONSTER_DEAD,
        /* NA_SE_EN_GOMA_UNARI */               SEQ_NOCAT,
        /* NA_SE_EN_GOMA_BJR_EGG1 */            SEQ_NOCAT,
        /* NA_SE_EN_GOMA_BJR_EGG2 */            SEQ_NOCAT,
        /* NA_SE_EN_GOMA_BJR_WALK */            SEQ_WALK,
        /* NA_SE_EN_GOMA_BJR_CRY */             SEQ_MONSTER_CRY_SHORT,
        /* NA_SE_EN_GOMA_BJR_DAM1 */            SEQ_NOCAT,
        /* NA_SE_EN_GOMA_BJR_DAM2 */            SEQ_NOCAT,
        /* NA_SE_EN_GOMA_BJR_DEAD */            SEQ_MONSTER_DEAD,
        /* NA_SE_EN_GOMA_DEMO_EYE */            SEQ_NOCAT,
        /* NA_SE_EN_GOMA_LAST */                SEQ_NOCAT,
        /* NA_SE_EN_GOMA_UNARI2 */              SEQ_NOCAT,
        /* NA_SE_EN_GOMA_FAINT */               SEQ_NOCAT,
        /* NA_SE_EN_GOMA_BJR_FREEZE */          SEQ_NOCAT,
        /* NA_SE_EN_DODO_M_CRY */               SEQ_MONSTER_CRY_MEDIUM,
        /* NA_SE_EN_DODO_M_DEAD */              SEQ_MONSTER_DEAD,
        /* NA_SE_EN_DODO_M_MOVE */              SEQ_NOCAT,
        /* NA_SE_EN_DODO_M_DOWN */              SEQ_NOCAT,
        /* NA_SE_EN_DODO_M_UP */                SEQ_NOCAT,
        /* NA_SE_EN_GANON_THROW_MASIC */        SEQ_NOCAT,
        /* NA_SE_EN_DODO_M_EAT */               SEQ_NOCAT,
        /* NA_SE_EN_GANON_DD_THUNDER */         SEQ_NOCAT,
        /* NA_SE_EN_RIZA_ONGND */               SEQ_NOCAT,
        /* NA_SE_EN_RIZA_CRY */                 SEQ_MONSTER_CRY_SHORT,
        /* NA_SE_EN_RIZA_ATTACK */              SEQ_MONSTER_ATTACK,
        /* NA_SE_EN_RIZA_DAMAGE */              SEQ_MONSTER_DAMAGED,
        /* NA_SE_EN_RIZA_WARAU */               SEQ_MONSTER_CRY_SHORT,
        /* NA_SE_EN_RIZA_DEAD */                SEQ_MONSTER_DEAD,
        /* NA_SE_EN_RIZA_WALK */                SEQ_WALK,
        /* NA_SE_EN_RIZA_JUMP */                SEQ_JUMP,
        /* NA_SE_EN_STALKID_WALK */             SEQ_WALK,
        /* NA_SE_EN_STALKID_ATTACK */           SEQ_MONSTER_ATTACK,
        /* NA_SE_EN_STALKID_DAMAGE */           SEQ_MONSTER_DAMAGED,
        /* NA_SE_EN_STALKID_DEAD */             SEQ_MONSTER_DEAD,
        /* NA_SE_EN_FLOORMASTER_SLIDING */      SEQ_NOCAT,
        /* NA_SE_EN_TEKU_WALK_WATER */          SEQ_NOCAT,
        /* NA_SE_EN_LIGHT_ARROW_HIT */          SEQ_NOCAT,
        /* NA_SE_EN_TUBOOCK_FLY */              SEQ_THROW_LOOP,
        /* NA_SE_EN_STAL_WARAU */               SEQ_MONSTER_CRY_MEDIUM,
        /* NA_SE_EN_STAL_SAKEBI */              SEQ_NOCAT,
        /* NA_SE_EN_STAL_DAMAGE */              SEQ_MONSTER_DAMAGED,
        /* NA_SE_EN_STAL_DEAD */                SEQ_MONSTER_DEAD,
        /* NA_SE_EN_WOLFOS_APPEAR */            SEQ_NOCAT,
        /* NA_SE_EN_STAL_WALK */                SEQ_NOCAT,
        /* NA_SE_EN_WOLFOS_CRY */               SEQ_MONSTER_CRY_MEDIUM,
        /* NA_SE_EN_WOLFOS_ATTACK */            SEQ_MONSTER_ATTACK,
        /* NA_SE_EN_FFLY_ATTACK */              SEQ_MONSTER_ATTACK,
        /* NA_SE_EN_FFLY_FLY */                 SEQ_NOCAT,
        /* NA_SE_EN_FFLY_DEAD */                SEQ_MONSTER_DEAD,
        /* NA_SE_EN_WOLFOS_DAMAGE */            SEQ_MONSTER_DAMAGED,
        /* NA_SE_EN_AMOS_WALK */                SEQ_WALK_LOUD,
        /* NA_SE_EN_AMOS_WAVE */                SEQ_NOCAT,
        /* NA_SE_EN_AMOS_DEAD */                SEQ_MONSTER_DEAD,
        /* NA_SE_EN_AMOS_DAMAGE */              SEQ_MONSTER_DAMAGED,
        /* NA_SE_EN_AMOS_VOICE */               SEQ_NOCAT,
        /* NA_SE_EN_SHELL_MOUTH */              SEQ_NOCAT,
        /* NA_SE_EN_SHELL_DEAD */               SEQ_MONSTER_DEAD,
        /* NA_SE_EN_WOLFOS_DEAD */              SEQ_MONSTER_DEAD,
        /* NA_SE_EN_DODO_K_COLI */              SEQ_NOCAT,
        /* NA_SE_EN_DODO_K_COLI2 */             SEQ_NOCAT,
        /* NA_SE_EN_DODO_K_ROLL */              SEQ_NOCAT,
        /* NA_SE_EN_DODO_K_BREATH */            SEQ_NOCAT,
        /* NA_SE_EN_DODO_K_DRINK */             SEQ_NOCAT,
        /* NA_SE_EN_DODO_K_DOWN */              SEQ_NOCAT,
        /* NA_SE_EN_DODO_K_OTAKEBI */           SEQ_NOCAT,
        /* NA_SE_EN_DODO_K_END */               SEQ_NOCAT,
        /* NA_SE_EN_DODO_K_LAST */              SEQ_NOCAT,
        /* NA_SE_EN_DODO_K_LAVA */              SEQ_NOCAT,
        /* NA_SE_EN_GANON_FLOAT */              SEQ_NOCAT,
        /* NA_SE_EN_GANON_DARKWAVE_M */         SEQ_NOCAT,
        /* NA_SE_EN_DODO_J_BREATH */            SEQ_NOCAT,
        /* NA_SE_EN_DODO_J_TAIL */              SEQ_NOCAT,
        /* NA_SE_EN_WOLFOS_WALK */              SEQ_WALK,
        /* NA_SE_EN_DODO_J_EAT */               SEQ_NOCAT,
        /* NA_SE_EN_DEKU_MOUTH */               SEQ_NOCAT,
        /* NA_SE_EN_DEKU_ATTACK */              SEQ_MONSTER_ATTACK,
        /* NA_SE_EN_DEKU_DAMAGE */              SEQ_MONSTER_DAMAGED,
        /* NA_SE_EN_DEKU_DEAD */                SEQ_MONSTER_DEAD,
        /* NA_SE_EN_DEKU_JR_MOUTH */            SEQ_NOCAT,
        /* NA_SE_EN_DEKU_JR_ATTACK */           SEQ_MONSTER_ATTACK,
        /* NA_SE_EN_DEKU_JR_DEAD */             SEQ_MONSTER_DEAD,
        /* NA_SE_EN_DEKU_SCRAPE */              SEQ_NOCAT,
        /* NA_SE_EN_TAIL_FLY */                 SEQ_NOCAT,
        /* NA_SE_EN_TAIL_CRY */                 SEQ_NOCAT,
        /* NA_SE_EN_TAIL_DEAD */                SEQ_MONSTER_DEAD,
        /* NA_SE_EN_GANON_SPARK */              SEQ_NOCAT,
        /* NA_SE_EN_STALTU_DOWN */              SEQ_NOCAT,
        /* NA_SE_EN_STALTU_UP */                SEQ_NOCAT,
        /* NA_SE_EN_STALTU_LAUGH */             SEQ_NOCAT,
        /* NA_SE_EN_STALTU_DAMAGE */            SEQ_MONSTER_DAMAGED,
        /* NA_SE_EN_STAL_JUMP */                SEQ_JUMP,
        /* NA_SE_EN_TEKU_DAMAGE */              SEQ_MONSTER_DAMAGED,
        /* NA_SE_EN_TEKU_DEAD */                SEQ_MONSTER_DEAD,
        /* NA_SE_EN_TEKU_WALK */                SEQ_WALK,
        /* NA_SE_EN_PO_KANTERA */               SEQ_NOCAT,
        /* NA_SE_EN_PO_FLY */                   SEQ_NOCAT,
        /* NA_SE_EN_PO_AWAY */                  SEQ_NOCAT,
        /* NA_SE_EN_PO_APPEAR */                SEQ_NOCAT,
        /* NA_SE_EN_PO_DISAPPEAR */             SEQ_NOCAT,
        /* NA_SE_EN_PO_DAMAGE */                SEQ_MONSTER_DAMAGED,
        /* NA_SE_EN_PO_DEAD */                  SEQ_MONSTER_DEAD,
        /* NA_SE_EN_PO_DEAD2 */                 SEQ_MONSTER_DEAD,
        /* NA_SE_EN_EXTINCT */                  SEQ_NOCAT,
        /* NA_SE_EN_GOLON_LAND_BIG */           SEQ_NOCAT,
        /* NA_SE_EN_RIZA_DOWN */                SEQ_NOCAT,
        /* NA_SE_EN_DODO_M_GND */               SEQ_NOCAT,
        /* NA_SE_EN_NUTS_UP */                  SEQ_NOCAT,
        /* NA_SE_EN_NUTS_DOWN */                SEQ_NOCAT,
        /* NA_SE_EN_NUTS_THROW */               SEQ_NOCAT,
        /* NA_SE_EN_NUTS_WALK */                SEQ_WALK,
        /* NA_SE_EN_NUTS_DAMAGE */              SEQ_MONSTER_DAMAGED,
        /* NA_SE_EN_NUTS_DEAD */                SEQ_MONSTER_DEAD,
        /* NA_SE_EN_NUTS_FAINT */               SEQ_NOCAT,
        /* NA_SE_EN_PO_BIG_GET */               SEQ_NOCAT,
        /* NA_SE_EN_STALTU_ROLL */              SEQ_NOCAT,
        /* NA_SE_EN_STALWALL_DEAD */            SEQ_MONSTER_DEAD,
        /* NA_SE_EN_PO_SISTER_DEAD */           SEQ_MONSTER_DEAD,
        /* NA_SE_EN_BARI_SPLIT */               SEQ_NOCAT,
        /* NA_SE_EN_TEKU_REVERSE */             SEQ_NOCAT,
        /* NA_SE_EN_VALVAISA_LAND2 */           SEQ_NOCAT,
        /* NA_SE_EN_TEKU_LAND_WATER */          SEQ_NOCAT,
        /* NA_SE_EN_LAST_DAMAGE */              SEQ_NOCAT,
        /* NA_SE_EN_STALWALL_ROLL */            SEQ_NOCAT,
        /* NA_SE_EN_STALWALL_DASH */            SEQ_NOCAT,
        /* NA_SE_EN_TEKU_JUMP_WATER */          SEQ_NOCAT,
        /* NA_SE_EN_TEKU_LAND_WATER2 */         SEQ_NOCAT,
        /* NA_SE_EN_FALL_AIM */                 SEQ_NOCAT,
        /* NA_SE_EN_FALL_UP */                  SEQ_NOCAT,
        /* NA_SE_EN_FALL_CATCH */               SEQ_NOCAT,
        /* NA_SE_EN_FALL_LAND */                SEQ_NOCAT,
        /* NA_SE_EN_FALL_WALK */                SEQ_WALK,
        /* NA_SE_EN_FALL_DAMAGE */              SEQ_MONSTER_DAMAGED,
        /* NA_SE_EN_FALL_DEAD */                SEQ_MONSTER_DEAD,
        /* NA_SE_EN_KAICHO_FLUTTER */           SEQ_NOCAT,
        /* NA_SE_EN_BIRI_FLY */                 SEQ_NOCAT,
        /* NA_SE_EN_BIRI_JUMP */                SEQ_NOCAT,
        /* NA_SE_EN_BIRI_SPARK */               SEQ_NOCAT,
        /* NA_SE_EN_BIRI_DEAD */                SEQ_MONSTER_DEAD,
        /* NA_SE_EN_BIRI_BUBLE */               SEQ_NOCAT,
        /* NA_SE_EN_BARI_ROLL */                SEQ_NOCAT,
        /* NA_SE_EN_GOMA_JR_FREEZE */           SEQ_NOCAT,
        /* NA_SE_EN_BARI_DEAD */                SEQ_MONSTER_DEAD,
        /* NA_SE_EN_GANON_FIRE */               SEQ_NOCAT,
        /* NA_SE_EN_FANTOM_TRANSFORM */         SEQ_NOCAT,
        /* NA_SE_EN_FANTOM_THUNDER */           SEQ_NOCAT,
        /* NA_SE_EN_FANTOM_SPARK */             SEQ_NOCAT,
        /* NA_SE_EN_FANTOM_FLOAT */             SEQ_NOCAT,
        /* NA_SE_EN_FANTOM_MASIC1 */            SEQ_NOCAT,
        /* NA_SE_EN_FANTOM_MASIC2 */            SEQ_NOCAT,
        /* NA_SE_EN_FANTOM_FIRE */              SEQ_NOCAT,
        /* NA_SE_EN_FANTOM_HIT_THUNDER */       SEQ_NOCAT,
        /* NA_SE_EN_FANTOM_ATTACK */            SEQ_NOCAT,
        /* NA_SE_EN_FANTOM_STICK */             SEQ_NOCAT,
        /* NA_SE_EN_FANTOM_EYE */               SEQ_NOCAT,
        /* NA_SE_EN_FANTOM_LAST */              SEQ_NOCAT,
        /* NA_SE_EN_FANTOM_THUNDER_GND */       SEQ_NOCAT,
        /* NA_SE_EN_FANTOM_DAMAGE */            SEQ_MONSTER_DAMAGED,
        /* NA_SE_EN_FANTOM_DEAD */              SEQ_MONSTER_DEAD,
        /* NA_SE_EN_FANTOM_LAUGH */             SEQ_VOICE_LONG,
        /* NA_SE_EN_FANTOM_DAMAGE2 */           SEQ_MONSTER_DAMAGED,
        /* NA_SE_EN_FANTOM_VOICE */             SEQ_NOCAT,
        /* NA_SE_EN_KAICHO_DAMAGE */            SEQ_MONSTER_DAMAGED,
        /* NA_SE_EN_GANON_ATTACK_DEMO */        SEQ_NOCAT,
        /* NA_SE_EN_GANON_FIRE_DEMO */          SEQ_NOCAT,
        /* NA_SE_EN_KAICHO_CRY */               SEQ_MONSTER_CRY_SHORT,
        /* NA_SE_EN_KAICHO_ATTACK */            SEQ_MONSTER_ATTACK,
        /* NA_SE_EN_MORIBLIN_WALK */            SEQ_WALK,
        /* NA_SE_EN_MORIBLIN_SLIDE */           SEQ_NOCAT,
        /* NA_SE_EN_MORIBLIN_ATTACK */          SEQ_MONSTER_ATTACK,
        /* NA_SE_EN_MORIBLIN_VOICE */           SEQ_MONSTER_CRY_SHORT,
        /* NA_SE_EN_MORIBLIN_SPEAR_AT */        SEQ_NOCAT,
        /* NA_SE_EN_MORIBLIN_SPEAR_NORM */      SEQ_NOCAT,
        /* NA_SE_EN_MORIBLIN_DEAD */            SEQ_MONSTER_DEAD,
        /* NA_SE_EN_MORIBLIN_DASH */            SEQ_NOCAT,
        /* NA_SE_EN_OCTAROCK_ROCK */            SEQ_NOCAT,
        /* NA_SE_EN_OCTAROCK_FLOAT */           SEQ_NOCAT,
        /* NA_SE_EN_OCTAROCK_JUMP */            SEQ_NOCAT,
        /* NA_SE_EN_OCTAROCK_LAND */            SEQ_NOCAT,
        /* NA_SE_EN_OCTAROCK_SINK */            SEQ_NOCAT,
        /* NA_SE_EN_OCTAROCK_BUBLE */           SEQ_NOCAT,
        /* NA_SE_EN_OCTAROCK_DEAD1 */           SEQ_MONSTER_DEAD,
        /* NA_SE_EN_OCTAROCK_DEAD2 */           SEQ_NOCAT,
        /* NA_SE_EN_BUBLE_WING */               SEQ_NOCAT,
        /* NA_SE_EN_BUBLE_MOUTH */              SEQ_NOCAT,
        /* NA_SE_EN_BUBLE_LAUGH */              SEQ_MONSTER_CRY_MEDIUM,
        /* NA_SE_EN_BUBLE_BITE */               SEQ_NOCAT,
        /* NA_SE_EN_BUBLE_UP */                 SEQ_NOCAT,
        /* NA_SE_EN_BUBLE_DOWN */               SEQ_NOCAT,
        /* NA_SE_EN_BUBLE_DEAD */               SEQ_MONSTER_DEAD,
        /* NA_SE_EN_BUBLEFALL_FIRE */           SEQ_NOCAT,
        /* NA_SE_EN_VALVAISA_APPEAR */          SEQ_NOCAT,
        /* NA_SE_EN_VALVAISA_ROAR */            SEQ_NOCAT,
        /* NA_SE_EN_VALVAISA_MAHI1 */           SEQ_NOCAT,
        /* NA_SE_EN_VALVAISA_MAHI2 */           SEQ_NOCAT,
        /* NA_SE_EN_VALVAISA_KNOCKOUT */        SEQ_NOCAT,
        /* NA_SE_EN_VALVAISA_DAMAGE1 */         SEQ_MONSTER_DAMAGED,
        /* NA_SE_EN_VALVAISA_DAMAGE2 */         SEQ_MONSTER_DAMAGED,
        /* NA_SE_EN_VALVAISA_ROCK */            SEQ_NOCAT,
        /* NA_SE_EN_VALVAISA_SW_NAIL */         SEQ_NOCAT,
        /* NA_SE_EN_VALVAISA_DEAD */            SEQ_MONSTER_DEAD,
        /* NA_SE_EN_VALVAISA_BURN */            SEQ_NOCAT,
        /* NA_SE_EN_VALVAISA_FIRE */            SEQ_NOCAT,
        /* NA_SE_EN_BARI_DAMAGE */              SEQ_MONSTER_DAMAGED,
        /* NA_SE_EN_MOFER_CORE_LAND */          SEQ_NOCAT,
        /* NA_SE_EN_MOFER_CORE_MOVE_WT */       SEQ_NOCAT,
        /* NA_SE_EN_MOFER_CORE_SMJUMP */        SEQ_NOCAT,
        /* NA_SE_EN_MONBLIN_GNDWAVE */          SEQ_NOCAT,
        /* NA_SE_EN_MONBLIN_HAM_DOWN */         SEQ_NOCAT,
        /* NA_SE_EN_MONBLIN_HAM_UP */           SEQ_NOCAT,
        /* NA_SE_EN_BUBLE_DAMAGE */             SEQ_MONSTER_DAMAGED,
        /* NA_SE_EN_REDEAD_CRY */               SEQ_MONSTER_CRY_LONG,
        /* NA_SE_EN_REDEAD_AIM */               SEQ_MONSTER_CRY_MEDIUM,
        /* NA_SE_EN_REDEAD_DAMAGE */            SEQ_MONSTER_DAMAGED,
        /* NA_SE_EN_REDEAD_DEAD */              SEQ_MONSTER_DEAD,
        /* NA_SE_EN_REDEAD_ATTACK */            SEQ_MONSTER_ATTACK,
        /* NA_SE_EN_NYU_MOVE */                 SEQ_NOCAT,
        /* NA_SE_EN_NYU_HIT_STOP */             SEQ_NOCAT,
        /* NA_SE_EN_KAICHO_DEAD */              SEQ_MONSTER_DEAD,
        /* NA_SE_EN_PO_LAUGH */                 SEQ_VOICE_MEDIUM,
        /* NA_SE_EN_PO_CRY */                   SEQ_MONSTER_CRY_MEDIUM,
        /* NA_SE_EN_PO_ROLL */                  SEQ_NOCAT,
        /* NA_SE_EN_PO_LAUGH2 */                SEQ_VOICE_MEDIUM,
        /* NA_SE_EN_MOFER_APPEAR */             SEQ_NOCAT,
        /* NA_SE_EN_MOFER_ATTACK */             SEQ_NOCAT,
        /* NA_SE_EN_MOFER_WAVE */               SEQ_NOCAT,
        /* NA_SE_EN_MOFER_CATCH */              SEQ_NOCAT,
        /* NA_SE_EN_MOFER_CUT */                SEQ_NOCAT,
        /* NA_SE_EN_MOFER_MOVE_DEMO */          SEQ_NOCAT,
        /* NA_SE_EN_MOFER_BUBLE_DEMO */         SEQ_NOCAT,
        /* NA_SE_EN_MOFER_CORE_JUMP */          SEQ_NOCAT,
        /* NA_SE_EN_MOFER_DEAD */               SEQ_MONSTER_DEAD,
        /* NA_SE_EN_MOFER_LASTVOICE */          SEQ_NOCAT,
        /* NA_SE_EN_MOFER_CORE_ROLL */          SEQ_NOCAT,
        /* NA_SE_EN_MOFER_CORE_FLY */           SEQ_NOCAT,
        /* NA_SE_EN_GOLON_WAKE_UP */            SEQ_NOCAT,
        /* NA_SE_EN_GOLON_SIT_DOWN */           SEQ_NOCAT,
        /* NA_SE_EN_CHICKEN_FLUTTER */          SEQ_NOCAT,
        /* NA_SE_EN_DEKU_WAKEUP */              SEQ_NOCAT,
        /* NA_SE_EN_DEADHAND_BITE */            SEQ_NOCAT,
        /* NA_SE_EN_DEADHAND_WALK */            SEQ_NOCAT,
        /* NA_SE_EN_DEADHAND_GRIP */            SEQ_NOCAT,
        /* NA_SE_EN_DEADHAND_HAND_AT */         SEQ_NOCAT,
        /* NA_SE_EN_DAIOCTA_MAHI */             SEQ_NOCAT,
        /* NA_SE_EN_DAIOCTA_SPLASH */           SEQ_NOCAT,
        /* NA_SE_EN_DAIOCTA_VOICE */            SEQ_NOCAT,
        /* NA_SE_EN_DAIOCTA_DAMAGE */           SEQ_MONSTER_DAMAGED,
        /* NA_SE_EN_DAIOCTA_SINK */             SEQ_NOCAT,
        /* NA_SE_EN_DAIOCTA_DEAD */             SEQ_MONSTER_DEAD,
        /* NA_SE_EN_DAIOCTA_DEAD2 */            SEQ_MONSTER_DEAD,
        /* NA_SE_EN_GANON_HIT_THUNDER */        SEQ_NOCAT,
        /* NA_SE_EN_TWINROBA_APPEAR_MS */       SEQ_NOCAT,
        /* NA_SE_EN_TWINROBA_TRANSFORM */       SEQ_NOCAT,
        /* NA_SE_EN_TWINROBA_MS_FIRE */         SEQ_NOCAT,
        /* NA_SE_EN_TWINROBA_FIRE_EXP */        SEQ_NOCAT,
        /* NA_SE_EN_TWINROBA_POWERUP */         SEQ_NOCAT,
        /* NA_SE_EN_TWINROBA_SHOOT_FREEZE */    SEQ_NOCAT,
        /* NA_SE_EN_TWINROBA_MS_FREEZE */       SEQ_NOCAT,
        /* NA_SE_EN_TWINROBA_MASIC_SET */       SEQ_NOCAT,
        /* NA_SE_EN_TWINROBA_CUTBODY */         SEQ_NOCAT,
        /* NA_SE_EN_GANON_HIT_GND_IMP */        SEQ_NOCAT,
        /* NA_SE_EN_TWINROBA_DAMAGE_VOICE */    SEQ_MONSTER_DAMAGED,
        /* NA_SE_EN_TWINROBA_REFL_FIRE */       SEQ_NOCAT,
        /* NA_SE_EN_TWINROBA_REFL_FREEZE */     SEQ_NOCAT,
        /* NA_SE_EN_GANON_CUTBODY */            SEQ_NOCAT,
        /* NA_SE_EN_TWINROBA_YOUNG_DAMAGE */    SEQ_MONSTER_DAMAGED,
        /* NA_SE_EN_TWINROBA_YOUNG_DEAD */      SEQ_MONSTER_DEAD,
        /* NA_SE_EN_GOLON_EYE_BIG */            SEQ_NOCAT,
        /* NA_SE_EN_GOLON_GOOD_BIG */           SEQ_NOCAT,
        /* NA_SE_EN_TWINROBA_FB_FLY */          SEQ_NOCAT,
        /* NA_SE_EN_TWINROBA_FLY */             SEQ_NOCAT,
        /* NA_SE_EN_TWINROBA_UNARI */           SEQ_NOCAT,
        /* NA_SE_EN_TWINROBA_ROLL */            SEQ_NOCAT,
        /* NA_SE_EN_TWINROBA_SHOOT_FIRE */      SEQ_NOCAT,
        /* NA_SE_EN_TWINROBA_THROW_MASIC */     SEQ_NOCAT,
        /* NA_SE_EN_DARUNIA_HIT_BREAST */       SEQ_NOCAT,
        /* NA_SE_EN_DARUNIA_HIT_LINK */         SEQ_NOCAT,
        /* NA_SE_EN_OWL_FLUTTER */              SEQ_NOCAT,
        /* NA_SE_EN_VALVAISA_LAND */            SEQ_NOCAT,
        /* NA_SE_EN_IRONNACK_WALK */            SEQ_WALK_LOUD,
        /* NA_SE_EN_IRONNACK_SWING_AXE */       SEQ_NOCAT,
        /* NA_SE_EN_IRONNACK_ARMOR_DEMO */      SEQ_NOCAT,
        /* NA_SE_EN_IRONNACK_STAGGER_DEMO */    SEQ_NOCAT,
        /* NA_SE_EN_IRONNACK_ARMOR_OFF_DEMO */  SEQ_NOCAT,
        /* NA_SE_EN_IRONNACK_ARMOR_LAND1_DEMO */ SEQ_NOCAT,
        /* NA_SE_EN_IRONNACK_ARMOR_LAND2_DEMO */ SEQ_NOCAT,
        /* NA_SE_EN_IRONNACK_ARMOR_LAND3_DEMO */ SEQ_NOCAT,
        /* NA_SE_EN_FLOORMASTER_ATTACK */       SEQ_NOCAT,
        /* NA_SE_EN_FLOORMASTER_SM_WALK */      SEQ_WALK,
        /* NA_SE_EN_FLOORMASTER_SM_DEAD */      SEQ_MONSTER_DEAD,
        /* NA_SE_EN_FLOORMASTER_RESTORE */      SEQ_NOCAT,
        /* NA_SE_EN_FLOORMASTER_EXPAND */       SEQ_NOCAT,
        /* NA_SE_EN_FLOORMASTER_SPLIT */        SEQ_NOCAT,
        /* NA_SE_EN_FLOORMASTER_SM_STICK */     SEQ_NOCAT,
        /* NA_SE_EN_FLOORMASTER_SM_LAND */      SEQ_NOCAT,
        /* NA_SE_EN_IRONNACK_WAVE_DEMO */       SEQ_NOCAT,
        /* NA_SE_EN_IRONNACK_FINGER_DEMO */     SEQ_NOCAT,
        /* NA_SE_EN_IRONNACK_ARMOR_HIT */       SEQ_NOCAT,
        /* NA_SE_EN_NUTS_CUTBODY */             SEQ_NOCAT,
        /* NA_SE_EN_BALINADE_LEVEL */           SEQ_NOCAT,
        /* NA_SE_EN_BALINADE_DAMAGE */          SEQ_MONSTER_DAMAGED,
        /* NA_SE_EN_BALINADE_FAINT */           SEQ_NOCAT,
        /* NA_SE_EN_BALINADE_BREAK */           SEQ_NOCAT,
        /* NA_SE_EN_BALINADE_DEAD */            SEQ_MONSTER_DEAD,
        /* NA_SE_EN_BALINADE_STICK */           SEQ_NOCAT,
        /* NA_SE_EN_BALINADE_THUNDER */         SEQ_NOCAT,
        /* NA_SE_EN_BALINADE_BL_SPARK */        SEQ_NOCAT,
        /* NA_SE_EN_BALINADE_BL_DEAD */         SEQ_MONSTER_DEAD,
        /* NA_SE_EN_BALINADE_BREAK2 */          SEQ_NOCAT,
        /* NA_SE_EN_BALINADE_HIT_RINK */        SEQ_NOCAT,
        /* NA_SE_EN_GANON_WAVE_GND */           SEQ_NOCAT,
        /* NA_SE_EN_AWA_BOUND */                SEQ_NOCAT,
        /* NA_SE_EN_AWA_BREAK */                SEQ_NOCAT,
        /* NA_SE_EN_BROB_WAVE */                SEQ_NOCAT,
        /* NA_SE_EN_NYU_DEAD */                 SEQ_MONSTER_DEAD,
        /* NA_SE_EN_EIER_DAMAGE */              SEQ_MONSTER_DAMAGED,
        /* NA_SE_EN_EIER_DEAD */                SEQ_MONSTER_DEAD,
        /* NA_SE_EN_EIER_FLUTTER */             SEQ_NOCAT,
        /* NA_SE_EN_EIER_FLY */                 SEQ_NOCAT,
        /* NA_SE_EN_SHADEST_TAIKO_LOW */        SEQ_NOCAT,
        /* NA_SE_EN_SHADEST_TAIKO_HIGH */       SEQ_NOCAT,
        /* NA_SE_EN_SHADEST_CLAP */             SEQ_NOCAT,
        /* NA_SE_EN_SHADEST_FLY_ATTACK */       SEQ_MONSTER_ATTACK,
        /* NA_SE_EN_PIHAT_UP */                 SEQ_NOCAT,
        /* NA_SE_EN_PIHAT_FLY */                SEQ_THROW_LOOP,
        /* NA_SE_EN_PIHAT_DAMAGE */             SEQ_MONSTER_DAMAGED,
        /* NA_SE_EN_PIHAT_LAND */               SEQ_NOCAT,
        /* NA_SE_EN_BALINADE_HAND_DOWN */       SEQ_NOCAT,
        /* NA_SE_EN_BALINADE_HAND_UP */         SEQ_NOCAT,
        /* NA_SE_EN_BALINADE_HAND_DAMAGE */     SEQ_MONSTER_DAMAGED,
        /* NA_SE_EN_BALINADE_HAND_DEAD */       SEQ_MONSTER_DEAD,
        /* NA_SE_EN_GOMA_JR_WALK */             SEQ_WALK,
        /* NA_SE_EN_GOMA_JR_CRY */              SEQ_MONSTER_CRY_SHORT,
        /* NA_SE_EN_GOMA_JR_DAM1 */             SEQ_NOCAT,
        /* NA_SE_EN_GOMA_JR_DAM2 */             SEQ_NOCAT,
        /* NA_SE_EN_GOMA_JR_DEAD */             SEQ_MONSTER_DEAD,
        /* NA_SE_EN_GOMA_EGG1 */                SEQ_NOCAT,
        /* NA_SE_EN_GOMA_EGG2 */                SEQ_NOCAT,
        /* NA_SE_EN_GANON_BODY_SPARK */         SEQ_NOCAT,
        /* NA_SE_EN_SHADEST_HAND_WAVE */        SEQ_NOCAT,
        /* NA_SE_EN_SHADEST_CATCH */            SEQ_NOCAT,
        /* NA_SE_EN_SHADEST_LAND */             SEQ_NOCAT,
        /* NA_SE_EN_SHADEST_HAND_FLY */         SEQ_NOCAT,
        /* NA_SE_EN_SHADEST_SHAKEHAND */        SEQ_NOCAT,
        /* NA_SE_EN_SHADEST_DAMAGE */           SEQ_MONSTER_DAMAGED,
        /* NA_SE_EN_SHADEST_DAMAGE_HAND */      SEQ_MONSTER_DAMAGED,
        /* NA_SE_EN_SHADEST_DISAPPEAR */        SEQ_NOCAT,
        /* NA_SE_EN_GANON_CHARGE_MASIC */       SEQ_NOCAT,
        /* NA_SE_EN_GANON_THROW_BIG */          SEQ_NOCAT,
        /* NA_SE_EN_SHADEST_FREEZE */           SEQ_NOCAT,
        /* NA_SE_EN_SHADEST_DEAD */             SEQ_MONSTER_DEAD,
        /* NA_SE_EN_BIMOS_ROLL_HEAD */          SEQ_NOCAT,
        /* NA_SE_EN_BIMOS_LAZER */              SEQ_NOCAT,
        /* NA_SE_EN_BIMOS_LAZER_GND */          SEQ_NOCAT,
        /* NA_SE_EN_BIMOS_AIM */                SEQ_NOCAT,
        /* NA_SE_EN_BUBLEWALK_WALK */           SEQ_NOCAT,
        /* NA_SE_EN_BUBLEWALK_AIM */            SEQ_NOCAT,
        /* NA_SE_EN_BUBLEWALK_REVERSE */        SEQ_NOCAT,
        /* NA_SE_EN_BUBLEWALK_DAMAGE */         SEQ_MONSTER_DAMAGED,
        /* NA_SE_EN_BUBLEWALK_DEAD */           SEQ_MONSTER_DEAD,
        /* NA_SE_EN_YUKABYUN_FLY */             SEQ_THROW_LOOP,
        /* NA_SE_EN_FLAME_DAMAGE */             SEQ_MONSTER_DAMAGED,
        /* NA_SE_EN_TWINROBA_FLY_DEMO */        SEQ_NOCAT,
        /* NA_SE_EN_FLAME_KICK */               SEQ_NOCAT,
        /* NA_SE_EN_FLAME_RUN */                SEQ_NOCAT,
        /* NA_SE_EN_FLAME_ROLL */               SEQ_NOCAT,
        /* NA_SE_EN_FLAME_MAN_RUN */            SEQ_WALK,
        /* NA_SE_EN_FLAME_MAN_DAMAGE */         SEQ_MONSTER_DAMAGED,
        /* NA_SE_EN_FLAME_LAUGH */              SEQ_VOICE_LONG,
        /* NA_SE_EN_FLAME_MAN_SLIDE */          SEQ_NOCAT,
        /* NA_SE_EN_FLAME_FIRE_ATTACK */        SEQ_NOCAT,
        /* NA_SE_EN_PIHAT_SM_FLY */             SEQ_THROW_LOOP,
        /* NA_SE_EN_PIHAT_SM_DEAD */            SEQ_MONSTER_DEAD,
        /* NA_SE_EN_RIVA_APPEAR */              SEQ_NOCAT,
        /* NA_SE_EN_AKINDONUTS_HIDE */          SEQ_NOCAT,
        /* NA_SE_EN_RIVA_DAMAGE */              SEQ_MONSTER_DAMAGED,
        /* NA_SE_EN_RIVA_DEAD */                SEQ_MONSTER_DEAD,
        /* NA_SE_EN_RIVA_MOVE */                SEQ_NOCAT,
        /* NA_SE_EN_FLAME_MAN_SURP */           SEQ_NOCAT,
        /* NA_SE_EN_SHADEST_LAST */             SEQ_NOCAT,
        /* NA_SE_EN_SHADEST_MOVE */             SEQ_NOCAT,
        /* NA_SE_EN_SHADEST_PRAY */             SEQ_NOCAT,
        /* NA_SE_EN_MGANON_ROAR */              SEQ_NOCAT,
        /* NA_SE_EN_LIKE_WALK */                SEQ_NOCAT,
        /* NA_SE_EN_LIKE_UNARI */               SEQ_NOCAT,
        /* NA_SE_EN_LIKE_DRINK */               SEQ_NOCAT,
        /* NA_SE_EN_LIKE_EAT */                 SEQ_NOCAT,
        /* NA_SE_EN_LIKE_THROW */               SEQ_NOCAT,
        /* NA_SE_EN_LIKE_DAMAGE */              SEQ_MONSTER_DAMAGED,
        /* NA_SE_EN_LIKE_DEAD */                SEQ_MONSTER_DEAD,
        /* NA_SE_EN_MGANON_SWORD */             SEQ_NOCAT,
        /* NA_SE_EN_GERUDOFT_ATTACK */          SEQ_MONSTER_ATTACK,
        /* NA_SE_EN_GERUDOFT_DAMAGE */          SEQ_MONSTER_DAMAGED,
        /* NA_SE_EN_GERUDOFT_DEAD */            SEQ_MONSTER_DEAD,
        /* NA_SE_EN_MGANON_DAMAGE */            SEQ_MONSTER_DAMAGED,
        /* NA_SE_EN_ANUBIS_FIRE */              SEQ_NOCAT,
        /* NA_SE_EN_ANUBIS_FIREBOMB */          SEQ_NOCAT,
        /* NA_SE_EN_MGANON_DEAD1 */             SEQ_MONSTER_DEAD,
        /* NA_SE_EN_ANUBIS_DEAD */              SEQ_MONSTER_DEAD,
        /* NA_SE_EN_MUSI_LAND */                SEQ_NOCAT,
        /* NA_SE_EN_MGANON_DEAD2 */             SEQ_MONSTER_DEAD,
        /* NA_SE_EN_EIER_ATTACK */              SEQ_NOCAT,
        /* NA_SE_EN_EIER_CRY */                 SEQ_MONSTER_CRY_SHORT,
        /* NA_SE_EN_FREEZAD_BREATH */           SEQ_NOCAT,
        /* NA_SE_EN_FREEZAD_DAMAGE */           SEQ_MONSTER_DAMAGED,
        /* NA_SE_EN_FREEZAD_DEAD */             SEQ_MONSTER_DEAD,
        /* NA_SE_EN_DEADHAND_LAUGH */           SEQ_VOICE_LONG,
        /* NA_SE_EN_DEADHAND_HIDE */            SEQ_NOCAT,
        /* NA_SE_EN_DEADHAND_DAMAGE */          SEQ_MONSTER_DAMAGED,
        /* NA_SE_EN_DEADHAND_HAND_DEAD */       SEQ_MONSTER_DEAD,
        /* NA_SE_EN_DEADHAND_DEAD */            SEQ_MONSTER_DEAD,
        /* NA_SE_EN_IRONNACK_BREAK_PILLAR2 */   SEQ_NOCAT,
        /* NA_SE_EN_IRONNACK_BREAK_PILLAR */    SEQ_NOCAT,
        /* NA_SE_EN_IRONNACK_HIT_GND */         SEQ_NOCAT,
        /* NA_SE_EN_MGANON_BREATH */            SEQ_NOCAT,
        /* NA_SE_EN_TWINROBA_LAUGH */           SEQ_VOICE_LONG,
        /* NA_SE_EN_TWINROBA_LAUGH2 */          SEQ_VOICE_LONG,
        /* NA_SE_EN_DUMMY434 */                 SEQ_NOSHUFFLE,
        /* NA_SE_EN_TWINROBA_SHOOT_VOICE */     SEQ_NOCAT,
        /* NA_SE_EN_TWINROBA_SENSE */           SEQ_NOCAT,
        /* NA_SE_EN_TWINROBA_DIE */             SEQ_NOCAT,
        /* NA_SE_EN_DUMMY438 */                 SEQ_NOSHUFFLE,
        /* NA_SE_EN_TWINROBA_YOUNG_DAMAGE2 */   SEQ_MONSTER_DAMAGED,
        /* NA_SE_EN_TWINROBA_YOUNG_SHOOTVC */   SEQ_NOCAT,
        /* NA_SE_EN_TWINROBA_YOUNG_LAUGH */     SEQ_VOICE_MEDIUM,
        /* NA_SE_EN_DUMMY442 */                 SEQ_NOSHUFFLE,
        /* NA_SE_EN_TWINROBA_YOUNG_WINK */      SEQ_NOCAT,
        /* NA_SE_EN_DUMMY444 */                 SEQ_NOSHUFFLE,
        /* NA_SE_EN_DUMMY445 */                 SEQ_NOSHUFFLE,
        /* NA_SE_EN_IRONNACK_DAMAGE */          SEQ_MONSTER_DAMAGED,
        /* NA_SE_EN_IRONNACK_DASH */            SEQ_NOCAT,
        /* NA_SE_EN_IRONNACK_DEAD */            SEQ_MONSTER_DEAD,
        /* NA_SE_EN_IRONNACK_PULLOUT */         SEQ_NOCAT,
        /* NA_SE_EN_IRONNACK_WAKEUP */          SEQ_NOCAT,
        /* NA_SE_EN_DUMMY451 */                 SEQ_NOSHUFFLE,
        /* NA_SE_EN_DUMMY452 */                 SEQ_NOSHUFFLE,
        /* NA_SE_EN_DUMMY453 */                 SEQ_NOSHUFFLE,
        /* NA_SE_EN_GERUDOFT_BREATH */          SEQ_VOICE_MEDIUM,
        /* NA_SE_EN_GANON_LAUGH */              SEQ_VOICE_LONG,
        /* NA_SE_EN_GANON_VOICE_DEMO */         SEQ_VOICE_MEDIUM,
        /* NA_SE_EN_GANON_THROW */              SEQ_VOICE_MEDIUM,
        /* NA_SE_EN_GANON_AT_RETURN */          SEQ_VOICE_SHORT,
        /* NA_SE_EN_GANON_HIT_GND */            SEQ_VOICE_MEDIUM,
        /* NA_SE_EN_GANON_DAMAGE1 */            SEQ_VOICE_MEDIUM,
        /* NA_SE_EN_GANON_DAMAGE2 */            SEQ_VOICE_MEDIUM,
        /* NA_SE_EN_GANON_DOWN */               SEQ_VOICE_MEDIUM,
        /* NA_SE_EN_GANON_RESTORE */            SEQ_VOICE_MEDIUM,
        /* NA_SE_EN_GANON_DEAD */               SEQ_MONSTER_DEAD,
        /* NA_SE_EN_GANON_BREATH */             SEQ_VOICE_MEDIUM,
        /* NA_SE_EN_GANON_TOKETU */             SEQ_VOICE_MEDIUM,
        /* NA_SE_EN_GANON_CASBREAK */           SEQ_VOICE_LONG,
        /* NA_SE_EN_GANON_BIGMASIC */           SEQ_VOICE_MEDIUM,
        /* NA_SE_EN_GANON_DARKWAVE */           SEQ_VOICE_LONG,
        /* NA_SE_EN_FANTOM_ST_LAUGH */          SEQ_VOICE_MEDIUM,
        /* NA_SE_EN_MGANON_WALK */              SEQ_WALK_LOUD,
        /* NA_SE_EN_MGANON_STAND */             SEQ_NOCAT,
        /* NA_SE_EN_MGANON_UNARI */             SEQ_NOCAT,
        /* NA_SE_EN_STALGOLD_ROLL */            SEQ_NOCAT,
        /* NA_SE_EN_KDOOR_WAVE */               SEQ_NOCAT,
        /* NA_SE_EN_KDOOR_HIT */                SEQ_NOCAT,
        /* NA_SE_EN_KDOOR_BREAK */              SEQ_NOCAT,
        /* NA_SE_EN_KDOOR_HIT_GND */            SEQ_NOCAT,
        /* NA_SE_EN_MGANON_SWDIMP */            SEQ_NOCAT,
        /* NA_SE_EN_STALTU_WAVE */              SEQ_NOCAT,
        /* NA_SE_EN_STALTU_DOWN_SET */          SEQ_NOCAT,
        /* NA_SE_EN_DUMMY482 */                 SEQ_NOSHUFFLE,
        /* NA_SE_EN_GOMA_BJR_LAND */            SEQ_NOCAT,
        /* NA_SE_EN_GOMA_BJR_LAND2 */           SEQ_NOCAT,
        /* NA_SE_EN_GOMA_JR_LAND */             SEQ_NOCAT,
        /* NA_SE_EN_GOMA_JR_LAND2 */            SEQ_NOCAT,
        /* NA_SE_EN_TWINROBA_FIGHT */           SEQ_NOCAT,
        /* NA_SE_EN_PO_BIG_CRY */               SEQ_NOCAT,
        /* NA_SE_EN_MUSI_SINK */                SEQ_NOCAT,
        /* NA_SE_EN_STALGOLD_UP_CRY */          SEQ_NOCAT,
        /* NA_SE_EN_GOLON_CRY */                SEQ_MONSTER_CRY_MEDIUM,
        /* NA_SE_EN_MOFER_CORE_DAMAGE */        SEQ_MONSTER_DAMAGED,
        /* NA_SE_EN_DAIOCTA_LAND_WATER */       SEQ_NOCAT,
        /* NA_SE_EN_RIVA_BIG_APPEAR */          SEQ_NOCAT,
        /* NA_SE_EN_MONBLIN_HAM_LAND */         SEQ_NOCAT,
        /* NA_SE_EN_MUSI_WALK */                SEQ_WALK,
        /* NA_SE_EN_MIMICK_BREATH */            SEQ_NOCAT,
        /* NA_SE_EN_STALWALL_LAUGH */           SEQ_NOCAT,
        /* NA_SE_EN_TWINROBA_TRANSFORM2 */      SEQ_NOCAT,
        /* NA_SE_EN_KAICHO_PIYORI */            SEQ_NOCAT,
        /* NA_SE_EN_DODO_K_WALK_APPEAR */       SEQ_WALK_LOUD,
        /* NA_SE_EN_DODO_K_STOP */              SEQ_NOCAT,
        /* NA_SE_EN_TEKU_GND993 */              SEQ_NOCAT,
        /* NA_SE_EN_TEKU_JUMP993 */             SEQ_JUMP,
        /* NA_SE_EN_MORIBLIN_DEMO */            SEQ_NOCAT,
        /* NA_SE_EN_LEADED_WHITE */             SEQ_NOCAT,
        /* NA_SE_EN_RIVA_HIDE_NEW */            SEQ_NOCAT,
        /* NA_SE_EN_STALBABY_HIDE_NEW */        SEQ_NOCAT,
        /* NA_SE_EN_STAL_REBORN */              SEQ_NOCAT,
        /* NA_SE_EN_DAIKU_FOOT */               SEQ_NOCAT,
        /* NA_SE_EN_STALKID_DAMAGE_NEW */       SEQ_MONSTER_DAMAGED,
        /* NA_SE_EN_STALKID_DOWN_NEW */         SEQ_NOCAT,
        /* NA_SE_EN_BALINADE_ARM_LAND */        SEQ_NOCAT,
        /* NA_SE_EN_BALINADE_ARM_DEAD */        SEQ_MONSTER_DEAD,
        /* NA_SE_EN_READED_WALK */              SEQ_WALK,
        /* NA_SE_EN_YOBI14 */                   SEQ_NOSHUFFLE,
        /* NA_SE_EN_YOBI15 */                   SEQ_NOSHUFFLE,
        /* NA_SE_EN_YOBI16 */                   SEQ_NOSHUFFLE,
        /* NA_SE_EN_YOBI17 */                   SEQ_NOSHUFFLE,
        /* NA_SE_EN_YOBI18 */                   SEQ_NOSHUFFLE,
        /* NA_SE_EN_YOBI19 */                   SEQ_NOSHUFFLE,
        /* NA_SE_EN_YOBI20 */                   SEQ_NOSHUFFLE,
        /* NA_SE_EN_YOBI21 */                   SEQ_NOSHUFFLE,
        /* NA_SE_EN_YOBI22 */                   SEQ_NOSHUFFLE,
        /* NA_SE_EN_YOBI23 */                   SEQ_NOSHUFFLE,
        /* NA_SE_EN_YOBI24 */                   SEQ_NOSHUFFLE,
        /* NA_SE_EN_YOBI25 */                   SEQ_NOSHUFFLE,
        /* NA_SE_EN_YOBI26 */                   SEQ_NOSHUFFLE,
        /* NA_SE_EN_YOBI27 */                   SEQ_NOSHUFFLE,
        /* NA_SE_EN_YOBI28 */                   SEQ_NOSHUFFLE,
        /* NA_SE_EN_YOBI29 */                   SEQ_NOSHUFFLE,
        /* NA_SE_EN_YOBI30 */                   SEQ_NOSHUFFLE,
        /* NA_SE_SY_WIN_OPEN */                 SEQ_NOSHUFFLE,
        /* NA_SE_SY_WIN_CLOSE */                SEQ_NOSHUFFLE,
        /* NA_SE_SY_CORRECT_CHIME */            SEQ_NOSHUFFLE,
        /* NA_SE_SY_GET_RUPY */                 SEQ_NOSHUFFLE,
        /* NA_SE_SY_MESSAGE_WOMAN */            SEQ_NOSHUFFLE,
        /* NA_SE_SY_MESSAGE_MAN */              SEQ_NOSHUFFLE,
        /* NA_SE_SY_ERROR */                    SEQ_NOSHUFFLE,
        /* NA_SE_SY_TRE_BOX_APPEAR */           SEQ_NOSHUFFLE,
        /* NA_SE_SY_DECIDE */                   SEQ_NOSHUFFLE,
        /* NA_SE_SY_CURSOR */                   SEQ_NOSHUFFLE,
        /* NA_SE_SY_CANCEL */                   SEQ_NOSHUFFLE,
        /* NA_SE_SY_HP_RECOVER */               SEQ_NOSHUFFLE,
        /* NA_SE_SY_ATTENTION_ON */             SEQ_NOSHUFFLE,
        /* NA_SE_SY_DUMMY_13 */                 SEQ_NOSHUFFLE,
        /* NA_SE_SY_DUMMY_14 */                 SEQ_NOSHUFFLE,
        /* NA_SE_SY_LOCK_OFF */                 SEQ_NOSHUFFLE,
        /* NA_SE_SY_LOCK_ON_HUMAN */            SEQ_NOSHUFFLE,
        /* NA_SE_SY_DUMMY_17 */                 SEQ_NOSHUFFLE,
        /* NA_SE_SY_DUMMY_18 */                 SEQ_NOSHUFFLE,
        /* NA_SE_SY_CAMERA_ZOOM_UP */           SEQ_NOSHUFFLE,
        /* NA_SE_SY_CAMERA_ZOOM_DOWN */         SEQ_NOSHUFFLE,
        /* NA_SE_SY_DUMMY_21 */                 SEQ_NOSHUFFLE,
        /* NA_SE_SY_DUMMY_22 */                 SEQ_NOSHUFFLE,
        /* NA_SE_SY_ATTENTION_ON_OLD */         SEQ_NOSHUFFLE,
        /* NA_SE_SY_MESSAGE_PASS */             SEQ_NOSHUFFLE,
        /* NA_SE_SY_WARNING_COUNT_N */          SEQ_NOSHUFFLE,
        /* NA_SE_SY_WARNING_COUNT_E */          SEQ_NOSHUFFLE,
        /* NA_SE_SY_HITPOINT_ALARM */           SEQ_NOSHUFFLE,
        /* NA_SE_SY_DUMMY_28 */                 SEQ_NOSHUFFLE,
        /* NA_SE_SY_DEMO_CUT */                 SEQ_NOSHUFFLE,
        /* NA_SE_SY_NAVY_CALL */                SEQ_NOSHUFFLE,
        /* NA_SE_SY_GAUGE_UP */                 SEQ_NOSHUFFLE,
        /* NA_SE_SY_DUMMY_32 */                 SEQ_NOSHUFFLE,
        /* NA_SE_SY_DUMMY_33 */                 SEQ_NOSHUFFLE,
        /* NA_SE_SY_DUMMY_34 */                 SEQ_NOSHUFFLE,
        /* NA_SE_SY_PIECE_OF_HEART */           SEQ_NOSHUFFLE,
        /* NA_SE_SY_GET_ITEM */                 SEQ_NOSHUFFLE,
        /* NA_SE_SY_WIN_SCROLL_LEFT */          SEQ_NOSHUFFLE,
        /* NA_SE_SY_WIN_SCROLL_RIGHT */         SEQ_NOSHUFFLE,
        /* NA_SE_SY_OCARINA_ERROR */            SEQ_NOSHUFFLE,
        /* NA_SE_SY_CAMERA_ZOOM_UP_2 */         SEQ_NOSHUFFLE,
        /* NA_SE_SY_CAMERA_ZOOM_DOWN_2 */       SEQ_NOSHUFFLE,
        /* NA_SE_SY_GLASSMODE_ON */             SEQ_NOSHUFFLE,
        /* NA_SE_SY_GLASSMODE_OFF */            SEQ_NOSHUFFLE,
        /* NA_SE_SY_FOUND */                    SEQ_NOSHUFFLE,
        /* NA_SE_SY_HIT_SOUND */                SEQ_NOSHUFFLE,
        /* NA_SE_SY_MESSAGE_END */              SEQ_NOSHUFFLE,
        /* NA_SE_SY_RUPY_COUNT */               SEQ_NOSHUFFLE,
        /* NA_SE_SY_LOCK_ON */                  SEQ_NOSHUFFLE,
        /* NA_SE_SY_GET_BOXITEM */              SEQ_NOSHUFFLE,
        /* NA_SE_SY_WHITE_OUT_L */              SEQ_NOSHUFFLE,
        /* NA_SE_SY_WHITE_OUT_S */              SEQ_NOSHUFFLE,
        /* NA_SE_SY_WHITE_OUT_T */              SEQ_NOSHUFFLE,
        /* NA_SE_SY_START_SHOT */               SEQ_NOSHUFFLE,
        /* NA_SE_SY_METRONOME */                SEQ_NOSHUFFLE,
        /* NA_SE_SY_ATTENTION_URGENCY */        SEQ_NOSHUFFLE,
        /* NA_SE_SY_METRONOME_LV */             SEQ_NOSHUFFLE,
        /* NA_SE_SY_FSEL_CURSOR */              SEQ_NOSHUFFLE,
        /* NA_SE_SY_FSEL_DECIDE_S */            SEQ_NOSHUFFLE,
        /* NA_SE_SY_FSEL_DECIDE_L */            SEQ_NOSHUFFLE,
        /* NA_SE_SY_FSEL_CLOSE */               SEQ_NOSHUFFLE,
        /* NA_SE_SY_FSEL_ERROR */               SEQ_NOSHUFFLE,
        /* NA_SE_SY_SET_FIRE_ARROW */           SEQ_NOSHUFFLE,
        /* NA_SE_SY_SET_ICE_ARROW */            SEQ_NOSHUFFLE,
        /* NA_SE_SY_SET_LIGHT_ARROW */          SEQ_NOSHUFFLE,
        /* NA_SE_SY_SYNTH_MAGIC_ARROW */        SEQ_NOSHUFFLE,
        /* NA_SE_SY_METRONOME_2 */              SEQ_NOSHUFFLE,
        /* NA_SE_SY_KINSTA_MARK_APPEAR */       SEQ_NOSHUFFLE,
        /* NA_SE_SY_FIVE_COUNT_LUPY */          SEQ_NOSHUFFLE,
        /* NA_SE_SY_CARROT_RECOVER */           SEQ_NOSHUFFLE,
        /* NA_SE_EV_FAIVE_LUPY_COUNT */         SEQ_NOCAT,
        /* NA_SE_SY_DUMMY_71 */                 SEQ_NOSHUFFLE,
        /* NA_SE_SY_KANADE_ISHI */              SEQ_NOSHUFFLE,
        /* NA_SE_SY_NA_HELLO_2 */               SEQ_NOSHUFFLE,
        /* NA_SE_SY_MAGIC_SOUL_NORMAL */        SEQ_NOSHUFFLE,
        /* NA_SE_SY_MAGIC_SOUL_FLASH */         SEQ_NOSHUFFLE,
        /* NA_SE_SY_CANCEL_CHALLENGE */         SEQ_NOSHUFFLE,
        /* NA_SE_SY_DECIDE_CHALLENGE */         SEQ_NOSHUFFLE,
        /* NA_SE_SY_YOBI05 */                   SEQ_NOSHUFFLE,
        /* NA_SE_SY_YOBI06 */                   SEQ_NOSHUFFLE,
        /* NA_SE_SY_YOBI07 */                   SEQ_NOSHUFFLE,
        /* NA_SE_SY_YOBI08 */                   SEQ_NOSHUFFLE,
        /* NA_SE_SY_YOBI09 */                   SEQ_NOSHUFFLE,
        /* NA_SE_SY_YOBI10 */                   SEQ_NOSHUFFLE,
        /* NA_SE_SY_YOBI11 */                   SEQ_NOSHUFFLE,
        /* NA_SE_SY_YOBI12 */                   SEQ_NOSHUFFLE,
        /* NA_SE_SY_YOBI13 */                   SEQ_NOSHUFFLE,
        /* NA_SE_SY_YOBI14 */                   SEQ_NOSHUFFLE,
        /* NA_SE_SY_YOBI15 */                   SEQ_NOSHUFFLE,
        /* NA_SE_SY_YOBI16 */                   SEQ_NOSHUFFLE,
        /* NA_SE_SY_YOBI17 */                   SEQ_NOSHUFFLE,
        /* NA_SE_OC_OCARINA */                  SEQ_NOCAT,
        /* NA_SE_OC_ABYSS */                    SEQ_NOCAT,
        /* NA_SE_OC_DOOR_OPEN */                SEQ_NOCAT,
        /* NA_SE_OC_SECRET_WARP_IN */           SEQ_NOCAT,
        /* NA_SE_OC_SECRET_WARP_OUT */          SEQ_NOCAT,
        /* NA_SE_OC_SECRET_HOLE_OUT */          SEQ_NOCAT,
        /* NA_SE_OC_REVENGE */                  SEQ_NOCAT,
        /* NA_SE_OC_HINT_MOVIE */               SEQ_NOCAT,
        /* NA_SE_OC_HINT_MOVIE2_WHITE */        SEQ_NOCAT,
        /* NA_SE_OC_HINT_MOVIE_ZOOMIN */        SEQ_NOCAT,
        /* NA_SE_OC_HIBIKI_ISHI */              SEQ_NOCAT,
        /* NA_SE_VO_LI_SWORD_N */               SEQ_VOICE_SHORT,
        /* NA_SE_VO_LI_SWORD_L */               SEQ_VOICE_MEDIUM,
        /* NA_SE_VO_LI_LASH */                  SEQ_VOICE_SHORT,
        /* NA_SE_VO_LI_HANG */                  SEQ_VOICE_SHORT,
        /* NA_SE_VO_LI_CLIMB_END */             SEQ_VOICE_SHORT,
        /* NA_SE_VO_LI_DAMAGE_S */              SEQ_VOICE_SHORT,
        /* NA_SE_VO_LI_FREEZE */                SEQ_VOICE_SHORT,
        /* NA_SE_VO_LI_FALL_S */                SEQ_VOICE_SHORT,
        /* NA_SE_VO_LI_FALL_L */                SEQ_VOICE_LONG,
        /* NA_SE_VO_LI_BREATH_REST */           SEQ_VOICE_SHORT,
        /* NA_SE_VO_LI_BREATH_DRINK */          SEQ_VOICE_SHORT,
        /* NA_SE_VO_LI_DOWN */                  SEQ_VOICE_LONG,
        /* NA_SE_VO_LI_TAKEN_AWAY */            SEQ_VOICE_LONG,
        /* NA_SE_VO_LI_HELD */                  SEQ_VOICE_SHORT,
        /* NA_SE_VO_LI_SNEEZE */                SEQ_NOCAT,
        /* NA_SE_VO_LI_SWEAT */                 SEQ_NOCAT,
        /* NA_SE_VO_LI_DRINK */                 SEQ_NOCAT,
        /* NA_SE_VO_LI_RELAX */                 SEQ_VOICE_LONG,
        /* NA_SE_VO_LI_SWORD_PUTAWAY */         SEQ_NOCAT,
        /* NA_SE_VO_LI_GROAN */                 SEQ_VOICE_SHORT,
        /* NA_SE_VO_LI_AUTO_JUMP */             SEQ_VOICE_SHORT,
        /* NA_SE_VO_LI_MAGIC_NALE */            SEQ_VOICE_MEDIUM,
        /* NA_SE_VO_LI_SURPRISE */              SEQ_VOICE_SHORT,
        /* NA_SE_VO_LI_MAGIC_FROL */            SEQ_VOICE_MEDIUM,
        /* NA_SE_VO_LI_PUSH */                  SEQ_VOICE_SHORT,
        /* NA_SE_VO_LI_HOOKSHOT_HANG */         SEQ_VOICE_SHORT,
        /* NA_SE_VO_LI_LAND_DAMAGE_S */         SEQ_VOICE_MEDIUM,
        /* NA_SE_VO_LI_NULL_0x1b */             SEQ_VOICE_SHORT,
        /* NA_SE_VO_LI_MAGIC_ATTACK */          SEQ_VOICE_MEDIUM,
        /* NA_SE_VO_BL_DOWN */                  SEQ_VOICE_LONG,
        /* NA_SE_VO_LI_DEMO_DAMAGE */           SEQ_VOICE_MEDIUM,
        /* NA_SE_VO_LI_ELECTRIC_SHOCK_LV */     SEQ_NOCAT,
        /* NA_SE_VO_LI_SWORD_N_KID */           SEQ_VOICE_SHORT,
        /* NA_SE_VO_LI_ROLLING_CUT_KID */       SEQ_VOICE_MEDIUM,
        /* NA_SE_VO_LI_LASH_KID */              SEQ_VOICE_SHORT,
        /* NA_SE_VO_LI_HANG_KID */              SEQ_VOICE_SHORT,
        /* NA_SE_VO_LI_CLIMB_END_KID */         SEQ_VOICE_SHORT,
        /* NA_SE_VO_LI_DAMAGE_S_KID */          SEQ_VOICE_SHORT,
        /* NA_SE_VO_LI_FREEZE_KID */            SEQ_VOICE_MEDIUM,
        /* NA_SE_VO_LI_FALL_S_KID */            SEQ_VOICE_SHORT,
        /* NA_SE_VO_LI_FALL_L_KID */            SEQ_NOCAT,
        /* NA_SE_VO_LI_BREATH_REST_KID */       SEQ_VOICE_MEDIUM,
        /* NA_SE_VO_LI_BREATH_DRINK_KID */      SEQ_VOICE_SHORT,
        /* NA_SE_VO_LI_DOWN_KID */              SEQ_VOICE_LONG,
        /* NA_SE_VO_LI_TAKEN_AWAY_KID */        SEQ_VOICE_LONG,
        /* NA_SE_VO_LI_HELD_KID */              SEQ_VOICE_SHORT,
        /* NA_SE_VO_LI_SNEEZE_KID */            SEQ_NOCAT,
        /* NA_SE_VO_LI_SWEAT_KID */             SEQ_NOCAT,
        /* NA_SE_VO_LI_DRINK_KID */             SEQ_NOCAT,
        /* NA_SE_VO_LI_RELAX_KID */             SEQ_NOCAT,
        /* NA_SE_VO_LI_SWORD_PUTAWAY_KID */     SEQ_VOICE_SHORT,
        /* NA_SE_VO_LI_GROAN_KID */             SEQ_VOICE_LONG,
        /* NA_SE_VO_LI_AUTO_JUMP_KID */         SEQ_VOICE_SHORT,
        /* NA_SE_VO_LI_MAGIC_NALE_KID */        SEQ_VOICE_MEDIUM,
        /* NA_SE_VO_LI_SURPRISE_KID */          SEQ_VOICE_MEDIUM,
        /* NA_SE_VO_LI_MAGIC_FROL_KID */        SEQ_VOICE_MEDIUM,
        /* NA_SE_VO_LI_PUSH_KID */              SEQ_VOICE_SHORT,
        /* NA_SE_VO_LI_HOOKSHOT_HANG_KID */     SEQ_VOICE_SHORT,
        /* NA_SE_VO_LI_LAND_DAMAGE_S_KID */     SEQ_VOICE_MEDIUM,
        /* NA_SE_VO_LI_NULL_0x1b_KID */         SEQ_VOICE_SHORT,
        /* NA_SE_VO_LI_MAGIC_ATTACK_KID */      SEQ_VOICE_MEDIUM,
        /* NA_SE_VO_BL_DOWN_KID */              SEQ_VOICE_LONG,
        /* NA_SE_VO_LI_DEMO_DAMAGE_KID */       SEQ_NOCAT,
        /* NA_SE_VO_LI_ELECTRIC_SHOCK_LV_KID */ SEQ_NOCAT,
        /* NA_SE_VO_NAVY_ENEMY */               SEQ_NOCAT,
        /* NA_SE_VO_NAVY_HELLO */               SEQ_NOCAT,
        /* NA_SE_VO_NAVY_HEAR */                SEQ_NOCAT,
        /* NA_SE_VO_NAVY_CALL */                SEQ_NOCAT,
        /* NA_SE_VO_NA_HELLO_3 */               SEQ_NOCAT,
        /* NA_SE_VO_DUMMY_0x45 */               SEQ_NOSHUFFLE,
        /* NA_SE_VO_DUMMY_0x46 */               SEQ_NOSHUFFLE,
        /* NA_SE_VO_DUMMY_0x47 */               SEQ_NOSHUFFLE,
        /* NA_SE_VO_DUMMY_0x48 */               SEQ_NOSHUFFLE,
        /* NA_SE_VO_DUMMY_0x49 */               SEQ_NOSHUFFLE,
        /* NA_SE_VO_DUMMY_0x4a */               SEQ_NOSHUFFLE,
        /* NA_SE_VO_DUMMY_0x4b */               SEQ_NOSHUFFLE,
        /* NA_SE_VO_DUMMY_0x4c */               SEQ_NOSHUFFLE,
        /* NA_SE_VO_DUMMY_0x4d */               SEQ_NOSHUFFLE,
        /* NA_SE_VO_DUMMY_0x4e */               SEQ_NOSHUFFLE,
        /* NA_SE_VO_DUMMY_0x4f */               SEQ_NOSHUFFLE,
        /* NA_SE_VO_TA_SLEEP */                 SEQ_VOICE_LONG,
        /* NA_SE_VO_TA_SURPRISE */              SEQ_VOICE_MEDIUM,
        /* NA_SE_VO_TA_CRY_0 */                 SEQ_VOICE_MEDIUM,
        /* NA_SE_VO_TA_CRY_1 */                 SEQ_VOICE_LONG,
        /* NA_SE_VO_IN_CRY_0 */                 SEQ_VOICE_LONG,
        /* NA_SE_VO_IN_LOST */                  SEQ_VOICE_LONG,
        /* NA_SE_VO_IN_LASH_0 */                SEQ_VOICE_MEDIUM,
        /* NA_SE_VO_IN_LASH_1 */                SEQ_VOICE_SHORT,
        /* NA_SE_VO_FR_LAUGH_0 */               SEQ_VOICE_LONG,
        /* NA_SE_VO_FR_SMILE_0 */               SEQ_VOICE_LONG,
        /* NA_SE_VO_NB_AGONY */                 SEQ_VOICE_MEDIUM,
        /* NA_SE_VO_NB_CRY_0 */                 SEQ_VOICE_LONG,
        /* NA_SE_VO_NB_NOTICE */                SEQ_VOICE_SHORT,
        /* NA_SE_VO_NA_HELLO_0 */               SEQ_NOCAT,
        /* NA_SE_VO_NA_HELLO_1 */               SEQ_NOCAT,
        /* NA_SE_VO_NA_HELLO_2 */               SEQ_NOCAT,
        /* NA_SE_VO_RT_CRASH */                 SEQ_VOICE_SHORT,
        /* NA_SE_VO_RT_DISCOVER */              SEQ_VOICE_MEDIUM,
        /* NA_SE_VO_RT_FALL */                  SEQ_VOICE_LONG,
        /* NA_SE_VO_RT_LAUGH_0 */               SEQ_VOICE_MEDIUM,
        /* NA_SE_VO_RT_LIFT */                  SEQ_VOICE_SHORT,
        /* NA_SE_VO_RT_THROW */                 SEQ_VOICE_SHORT,
        /* NA_SE_VO_RT_UNBALLANCE */            SEQ_VOICE_MEDIUM,
        /* NA_SE_VO_ST_DAMAGE */                SEQ_VOICE_LONG,
        /* NA_SE_VO_ST_ATTACK */                SEQ_VOICE_MEDIUM,
        /* NA_SE_VO_Z0_HURRY */                 SEQ_VOICE_MEDIUM,
        /* NA_SE_VO_Z0_MEET */                  SEQ_VOICE_SHORT,
        /* NA_SE_VO_Z0_QUESTION */              SEQ_VOICE_SHORT,
        /* NA_SE_VO_Z0_SIGH_0 */                SEQ_VOICE_MEDIUM,
        /* NA_SE_VO_Z0_SMILE_0 */               SEQ_VOICE_MEDIUM,
        /* NA_SE_VO_Z0_SURPRISE */              SEQ_VOICE_MEDIUM,
        /* NA_SE_VO_Z0_THROW */                 SEQ_VOICE_MEDIUM,
        /* NA_SE_VO_SK_CRY_0 */                 SEQ_VOICE_MEDIUM,
        /* NA_SE_VO_SK_CRY_1 */                 SEQ_VOICE_MEDIUM,
        /* NA_SE_VO_SK_CRASH */                 SEQ_VOICE_SHORT,
        /* NA_SE_VO_SK_LAUGH */                 SEQ_NOCAT,
        /* NA_SE_VO_SK_SHOUT */                 SEQ_VOICE_SHORT,
        /* NA_SE_VO_Z1_CRY_0 */                 SEQ_VOICE_MEDIUM,
        /* NA_SE_VO_Z1_CRY_1 */                 SEQ_VOICE_MEDIUM,
        /* NA_SE_VO_Z1_OPENDOOR */              SEQ_VOICE_MEDIUM,
        /* NA_SE_VO_Z1_SURPRISE */              SEQ_VOICE_SHORT,
        /* NA_SE_VO_Z1_PAIN */                  SEQ_VOICE_MEDIUM,
        /* NA_SE_VO_KZ_MOVE */                  SEQ_VOICE_MEDIUM,
        /* NA_SE_VO_NB_LAUGH */                 SEQ_NOCAT,
        /* NA_SE_VO_IN_LAUGH */                 SEQ_NOCAT,
        /* NA_SE_VO_DUMMY_0x7d */               SEQ_NOSHUFFLE,
        /* NA_SE_VO_DUMMY_0x7e */               SEQ_NOSHUFFLE,
        /* NA_SE_VO_DUMMY_0x7f */               SEQ_NOSHUFFLE,
        /* NA_SE_VO_DUMMY_0x80 */               SEQ_NOSHUFFLE,
        /* NA_SE_VO_LI_AUTO_JUMP_DARKLINK */    SEQ_VOICE_SHORT,
    };

    static SFXData sfxData;

    const SFXData& GetSFXData() {
        return sfxData;
    }

    void InitSFXRandomizer() {
        // Set the sequences to default
        for (int i = 0; i < SFX_COUNT; i++) {
            sfxData.rSFXOverrides_All[i] = SFX_BASE + i;
        }
        // Set the amount of sequence types for each category
        for (size_t i = 0; i < SEQTYPE_COUNT; i++) {
            sfxData.rSeqMaxes[i] = 0;
        }
        for (size_t i = 0; i < seqTypesSFX.size(); i++) {
            if (seqTypesSFX[i] >= SEQTYPE_COUNT) {
                continue;
            }
            sfxData.rSeqMaxes[seqTypesSFX[i]]++;
        }
        // Set the sequence type list
        for (int i = 0; i < SFX_COUNT; i++) {
            sfxData.rSeqTypesSFX[i] = seqTypesSFX[i];
        }
    }

    void ShuffleSequences(bool shuffleCategorically) {
        std::vector<uint32_t> seqs;
        if (shuffleCategorically) {
            for (int type = 0; type < SEQTYPE_COUNT; type++) {
                // Get all sequences of the desired type into a vector
                for (int i = 0; i < SFX_COUNT; i++) {
                    if (seqTypesSFX[i] == type) {
                        seqs.push_back(sfxData.rSFXOverrides_All[i]);
                    }
                }

                // Shuffle the vector...
                for (size_t i = 0; i < seqs.size(); i++) {
                    std::swap(seqs[i], seqs[rand() % seqs.size()]);
                }

                // ...and feed it back into the overrides arrays
                for (size_t i = 0; i < seqs.size(); i++) {
                    sfxData.rSFXOverrides_Types[type][i] = seqs[i];
                }
                for (int i = 0; i < SFX_COUNT; i++) {
                    if (seqTypesSFX[i] == type) {
                        sfxData.rSFXOverrides_All[i] = seqs.back();
                        seqs.pop_back();
                    }
                }
            }
        } else {
            // Get all sequences into a vector
            for (int i = 0; i < SFX_COUNT; i++) {
                if (seqTypesSFX[i] == SEQ_NOSHUFFLE) {
                    continue;
                }
                seqs.push_back(sfxData.rSFXOverrides_All[i]);
            }

            // Shuffle the vector...
            for (size_t i = 0; i < seqs.size(); i++) {
                std::swap(seqs[i], seqs[rand() % seqs.size()]);
            }

            // ...and feed it back into the overrides arrays
            for (size_t i = 0; i < seqs.size(); i++) {
                sfxData.rSFXOverrides_AllTrimmed[i] = seqs[i];
            }
            for (int i = 0; i < SFX_COUNT; i++) {
                if (seqTypesSFX[i] == SEQ_NOSHUFFLE) {
                    continue;
                }
                sfxData.rSFXOverrides_All[i] = seqs.back();
                seqs.pop_back();
            }
        }
    }
} // namespace SFX
